//Some special functions in quantum mechanics
#include <iostream.h>
#include <math.h>

#include "userutil.h"
#include "ryspoly.h"
#include "gausquad.h"

Gausquad::Gausquad()
{ error=107; init();
}//end of Gausquad::Gausquad

Gausquad::~Gausquad()
{ if (talpha) delete [] talpha; if (walpha) delete [] walpha;
  if (ualpha) delete [] ualpha; if (quadint) delete [] quadint;
}//end of Gausquad::~Gausquad

void Gausquad::init()
{
  if ((error==107)||(error==103))
  { ndimt=0; talpha=walpha=quadint=NULL;
  }
  if (error==103)
  { talpha=new float [100]; walpha=new float [100];
    ualpha=new float [100];
    if ((!talpha)||(!walpha)||(!ualpha)) error=102;
  }
  else if (error==106)
  { if (talpha)
    { talpha=new float [100];
      if (!talpha) error=102;
    }
    if (walpha)
    { walpha=new float [100];
      if (!walpha) error=102;
    }
    if (ualpha)
    { ualpha=new float [100];
      if (!ualpha) error=102;
    }
    if (quadint)
    { quadint=new float [ndimt];
      if (!quadint) error=102;
    }
  }
  if (error==106) error=0;
  else if (error==103) error=101;
  else if (error==107) error=103;

  return;
} //end of Gausquad::init

int Gausquad::getlength()
{ int ka;
  if (error==0)
  { ka=sizeof(Gausquad)+sizeof(int);
    if (talpha) ka+=100*sizeof(float);
    if (walpha) ka+=100*sizeof(float);
    if (ualpha) ka+=100*sizeof(float);
    if (quadint) ka+=ndimt*sizeof(float);
  }
  else ka=0;
  return(ka);
} //end of Gausquad::getlength

int Gausquad::export(char *dest,int length)
{ int ka,kb;
  if ((error==0)&&(dest))
  { ka=0; kb=getlength();
    if (length==0) length=kb;
    else if (length!=kb) ka=-1;
    kb=sizeof(int);
    if (ka>=0) ka=memorysend(dest,(char *)&length,ka,kb,length);
    kb=sizeof(Gausquad);
    if (ka>=0) ka=memorysend(dest,(char *)this,ka,kb,length);
    kb=100*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)talpha,ka,kb,length);
    kb=100*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)walpha,ka,kb,length);
    kb=100*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)ualpha,ka,kb,length);
    kb=ndimt*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)quadint,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  return(error);
} //end of Gausquad::export

int Gausquad::import(char *source,int length)
{ int ka,kb;

  if ((error==103)&&(source))
  { memorycopy((char *)&kb,source,sizeof(int));
    ka=sizeof(int);
    if ((length==0)&&(kb>0)) length=kb;
    else if ((length<0)||(kb<0)||(length!=kb)) ka=-1;
    kb=sizeof(Gausquad);
    if (ka>=0) ka=memoryrec((char *)this,source,ka,kb,length);
    if (ka<0) error=901;
    else if (error==0)
    { error=106; init();
    }
    else error=754;
  }
  else ka=-1;
  if (error==0)
  { kb=100*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)talpha,source,ka,kb,length);
    kb=100*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)walpha,source,ka,kb,length);
    kb=100*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)ualpha,source,ka,kb,length);
    kb=ndimt*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)quadint,source,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }

  return(error);
} //end of Gausquad::import


/*
----------------------------------------------------------------------
list of parameters of paper:
  J. Rys, M. Dupuis, and H.F. King, J. Computational Chem. 4 (2),
    154-157 (1983)

in program     in paper         in program     in paper
   xa             xi               ya            B00
   xb             xj               yb            B10
   xc             xk               yc            B01'
   xd             xl               yd            C00
   xe             xA               ye            C00'
   xf             xB               yf          pi/(AB)1/2
   xg             Dx               yg          G00/yf
   xh             Gx               yh          (xA-xB)*B00*2
   xi           xi-xj
   xj           xk-xl

   ka             ai               na             ni
   kb             aj               nb             nj
   kc             ak               nc             nk
   kd             al               nd             nl
   ke          A=ai+aj             ndimd          1
   kf          B=ak+al             ndimc     max(ni+nj,nk+nl)
   kg          rho=AB/(A+B)        ndimb       ndimc*ndimc
   kh          aiaj/(ai+aj)        ndima       ndimb*ndimc
   ki          akal/(ak+al)        ndims       ndima*ndimc

----------------------------------------------------------------------
*/
int Gausquad::loadparas(int waiver,int *narray,float *xarray,
  float *karray,Ryspoly *ryspoly)
{ int ig,ih,ma,ne;
  float ka,kb,kc,kd,ke,kf,kg,kh,ki,xt,za;

  if ((error==101)||(error==0))
  { error=0;

    ne=1;
    for (ih=0;ih<12;ih+=2)
    { ma=narray[ih]+narray[ih+1]+1;
      if (ne<ma) ne=ma;
    }
    if (ne<=1) ndima=ndimb=ndimc=ndimd=ndims=nt=1;
    else
    { ndima=ne*ne*ne; ndimb=ne*ne; ndimc=ne; ndimd=1;
      ndims=ndima*ne;
      nt=0;
      for (ih=0;ih<12;++ih) nt+=narray[ih];
      nt>>=1; ++nt;
    }

    if ((ndimt<ndims*nt*4)||(!quadint))
    { ndimt=ndims*nt*4;
      if (quadint) delete [] quadint;
      quadint=new float [ndimt];
      if (!quadint) error=102;
    }
  }

  if (error==0)
  { ka=karray[0]; kb=karray[1]; kc=karray[2]; kd=karray[3];
    ke=ka+kb; kf=kc+kd; kg=ke*kf/(ke+kf);
    kh=ka*kb/(ka+kb); ki=kc*kd/(kc+kd);

    xt=0.0f;
    for (ih=0;ih<12;ih+=4)
    { za=(ka*xarray[ih+0]+kb*xarray[ih+1])/ke;
      za-=(kc*xarray[ih+2]+kd*xarray[ih+3])/kf;
      xt+=za*za;
    }
    xt*=kg;

    error=ryspoly->getalphas(nt,xt,talpha,walpha,ualpha);
  }

  if (error==0)
  { za=2.0*sqrt(kg/3.14159265);
    for (ig=0;ig<nt;++ig) walpha[ig]*=za;
  }

  makeintegral(waiver,narray,xarray,karray);

  return(error);
}//end of Gausquad::loadparas

int Gausquad::makeintegral(int waiver,int *narray,float *xarray,
  float *karray)
{ int ig,ih,ii,ij,nx;
  float za;

  if (error==0)
  { if (waiver&1) makexelem(0,narray,xarray,karray);
    if (waiver&2) makexelem(0,narray,xarray,karray);
    if (waiver&4) makexelem(0,narray,xarray,karray);
    ih=nt; ii=nt+nt; ij=nt+nt+nt; za=0.0f;
    for (ig=0;ig<nt;++ig)
      za+=quadint[ig+ih]*quadint[ig+ii]*quadint[ig+ij];
    quadint[0]=za;
  }

  return(error);
}//end of Gausquad::makeintegral

//nx=0,1,2
int Gausquad::makexelem(int nx,int *narray,float *xarray,float *karray)
{ int ia,ib,ic,id,ie,ig,ih,na,nb,nc,nd,ne,ma;
  float xa,xb,xc,xd,xe,xf,xg,xh,xi,xj,ya,yb,yc,yd,ye,yf,yh,za,ka,kb,
    kc,kd,ke,kf,kg,kh,ki;

  if (error==0)
  { ma=nx*4;
    na=narray[ma+0]+1; nb=narray[ma+1]+1; nc=narray[ma+2]+1;
    nd=narray[ma+3]+1;
    xa=xarray[ma+0]; xb=xarray[ma+1]; xc=xarray[ma+2]; xd=xarray[ma+3];
    ka=karray[0]; kb=karray[1]; kc=karray[2]; kd=karray[3];

    ne=na+nb-1;
    if (ne<nc+nd-1) ne=nc+nd-1;

    if (na<nb)
    { za=xa; xa=xb; xb=za;
      za=ka; ka=kb; kb=za;
      ma=na; na=nb; nb=ma;
    }
    if (nc<nd)
    { za=xc; xc=xd; xd=za;
      za=kc; kc=kd; kd=za;
      ma=nc; nc=nd; nd=ma;
    }

    ke=ka+kb; kf=kc+kd; kg=ke*kf/(ke+kf);
    kh=ka*kb/(ka+kb); ki=kc*kd/(kc+kd);
    xh=kh*(xa-xb)*(xa-xb)+ki*(xc-xd)*(xc-xd);

    if (ne<=1) xe=xf=xg=xi=xj=0.0f;
    else
    { xe=(ka*xa+kb*xb)/ke; xf=(kc*xc+kd*xd)/kf; xg=kg*(xe-xf)*(xe-xf);
      xi=xa-xb; xj=xc-xd;
    }
  }

  for (ig=0;(error==0)&&(ig<nt);++ig)
  { yf=3.1415926535f/sqrt(ke*kf);
    if (ne<=1)
    { ih=ig+(nx+1)*nt; za=xh*yf;
      if (nx==0) za*=walpha[ig];
      quadint[ih]=za;
    }
    else
    { ma=(ig+(nx+1)*nt)*ndims;
      ya=0.5f*talpha[ig]*talpha[ig]/(ke+kf);
      yb=(0.5f-ya*kf)/ke; yc=(0.5f-ya*ke)/kf;
      yh=(xe-xf)*(ya+ya);
      yd=xe-xa-kf*yh;
      ye=xf-xc+ke*yh;

      for (ie=0;ie<ne;++ie)
      { for (ia=ie;ia<na+nb-1;++ia)
        { ic=ie; ih=ma+ia*ndima+ic*ndimc;
          if ((ia==0)&&(ic==0))
          { za=xh*yf;
            if (nx==0) za*=walpha[ig];
          }
          else
          { za=yd*quadint[ih-ndima];
            if (ic>0) za+=ic*ya*quadint[ih-ndima-ndimc];
            if (ia>1) za+=(ia-1)*yb*quadint[ih-ndima-ndima];
          }
          quadint[ih]=za;
        }
        for (ic=ie+1;ic<nc+nd-1;++ic)
        { ia=ie; ih=ma+ia*ndima+ic*ndimc;
          za=ye*quadint[ih-ndimc];
          if (ia>0) za+=ia*ya*quadint[ih-ndima-ndimc];
          if (ic>1) za+=(ic-1)*yc*quadint[ih-ndimc-ndimc];
          quadint[ih]=za;
        }
      }

      for (ic=0;ic<nc+nd-1;++ic)
      { for (ib=1;ib<nb;++ib)
        { for (ia=0;ia<na+nb-1-ib;++ia)
          { ih=ma+ia*ndima+ib*ndimb+ic*ndimc;
            quadint[ih]=quadint[ih+ndima-ndimb]+
              xi*quadint[ih-ndimb];
          }
        }
      }

      ia=na-1; ib=nb-1;
      for (id=1;id<nd;++id)
      { for (ic=0;ic<nc+nd-1-id;++ic)
        { ih=ma+ia*ndima+ib*ndimb+ic*ndimc+id*ndimd;
          quadint[ih]=quadint[ih+ndimc-ndimd]+
            xj*quadint[ih-ndimd];
        }
      }
      ic=nc-1; id=nd-1;
      ih=ma+ia*ndima+ib*ndimb+ic*ndimc+id*ndimd;
      quadint[ig+(nx+1)*nt]=quadint[ih];
    }
  }
  return(error);
}//end of Gausquad::makexelem

float Gausquad::fquadint()
{ float za;
  if (error==0) za=quadint[0]; else za=0.0f;
  return(za);
}//end of Gausquad::fquadint

