#include <iostream.h>
#include <math.h>

#include "userutil.h"
#include "matrix.h"

Matrix::Matrix()
{ error=107; init();
}//end of Matrix::Matrix

Matrix::~Matrix()
{ if (origmat) delete [] origmat; if (lumat) delete [] lumat;
  if (invmat) delete [] invmat;
  if (pivot) delete [] pivot;
}//end of Matrix::~Matrix

void Matrix::init()
{
  if ((error==107)||(error==103))
  { elenum=eledim=exchange=0; origmat=lumat=invmat=NULL; pivot=NULL;
  }
  if (error==103)
  { elenum=eledim=exchange=0; origmat=lumat=invmat=NULL; pivot=NULL;
  }
  else if (error==106)
  { if (origmat)
    { origmat=new double [elenum];
      if (!origmat) error=102;
    }
    if (lumat)
    { lumat=new double [elenum];
      if (!lumat) error=102;
    }
    if (invmat)
    { invmat=new double [elenum];
      if (!invmat) error=102;
    }
    if (pivot)
    { pivot=new int [eledim];
      if (!pivot) error=102;
    }
  }
  if (error==106) error=0;
  else if (error==103) error=101;
  else if (error==107) error=103;

  return;
} //end of Matrix::init

int Matrix::getlength()
{ int ka;
  if (error==0)
  { ka=sizeof(Matrix)+sizeof(int);
    if (origmat) ka+=elenum*sizeof(double);
    if (lumat) ka+=elenum*sizeof(double);
    if (invmat) ka+=elenum*sizeof(double);
    if (pivot) ka+=eledim*sizeof(int);
  }
  else ka=0;
  return(ka);
} //end of Matrix::getlength

int Matrix::export(char *dest,int length)
{ int ka,kb;
  if ((error==0)&&(dest))
  { ka=0; kb=getlength();
    if (length==0) length=kb;
    else if (length!=kb) ka=-1;
    kb=sizeof(int);
    if (ka>=0) ka=memorysend(dest,(char *)&length,ka,kb,length);
    kb=sizeof(Matrix);
    if (ka>=0) ka=memorysend(dest,(char *)this,ka,kb,length);
    kb=elenum*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)origmat,ka,kb,length);
    kb=elenum*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)lumat,ka,kb,length);
    kb=elenum*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)invmat,ka,kb,length);
    kb=eledim*sizeof(int);
    if (ka>=0) ka=memorysend(dest,(char *)pivot,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  return(error);
} //end of Matrix::export

int Matrix::import(char *source,int length)
{ int ka,kb;

  if ((error==103)&&(source))
  { memorycopy((char *)&kb,source,sizeof(int));
    ka=sizeof(int);
    if ((length==0)&&(kb>0)) length=kb;
    else if ((length<0)||(kb<0)||(length!=kb)) ka=-1;
    kb=sizeof(Matrix);
    if (ka>=0) ka=memoryrec((char *)this,source,ka,kb,length);
    if (ka<0) error=901;
    else if (error==0)
    { error=106; init();
    }
    else error=754;
  }
  else ka=-1;
  if (error==0)
  { kb=elenum*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)origmat,source,ka,kb,length);
    kb=elenum*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)lumat,source,ka,kb,length);
    kb=elenum*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)invmat,source,ka,kb,length);
    kb=eledim*sizeof(int);
    if (ka>=0) ka=memoryrec((char *)pivot,source,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  return(error);
} //end of Matrix::import

int Matrix::allocation()
{ int sizeint;

  if (error==0) sizeint=sizeof(int); else sizeint=sizeof(int);
  if ((error==0)&&(eledim<0)) error=614;
  else if ((error==0)&&(sizeint<4)&&(eledim>64)) error=615;
  else if ((error==0)&&(eledim>1024)) error=615;
  else
  { if (origmat) delete [] origmat; if (lumat) delete [] lumat;
    if (invmat) delete [] invmat; if (pivot) delete [] pivot;
    elenum=eledim*eledim;
    origmat=new double [elenum]; lumat=new double [elenum];
    invmat=new double [elenum]; pivot=new int [eledim];
    if ((!origmat)||(!lumat)||(!invmat)||(!pivot)) error=102;
  }

  return(error);
} //end of Matrix::allocation

int Matrix::loadmatrix(int aeledim,double *tmat)
{ int i;

  if (((error==101)||(error==0))&&(tmat))
  { error=0; eledim=aeledim;
    error=allocation();
  }

  if ((error==0)&&(tmat))
  { for (i=0;i<elenum;++i) origmat[i]=lumat[i]=tmat[i];
    exchange=0;
  }

  return(error);
}// end of Matrix::loadmatrix

float Matrix::getmemory()
{ float xa;
  if (error==0)
    xa=(float)getlength();
  else xa=-1.0f;
  return(xa);
} //end of Matrix::getmemory

double Matrix::determinant(int lud)
{ int i,j,k,ma,mb;
  double xa,xb,xc,xd,coef,sum;

  if ((error==0)&&(lud==0))
  { exchange=1; coef=1.0f;

    for (k=0;(coef>1.0e-30)&&(k<eledim-1);++k)
    { xa=0.0f; ma=mb=k;
      for (i=k;i<eledim;++i)
      { xb=0.0f;
        for (j=k;j<eledim;++j)
        { if (k==0) xd=fabs(origmat[i*eledim+j]);
          else xd=fabs(lumat[i*eledim+j]);
          if (xb<xd) xb=xd;
          if (xa<xd)
          { xa=xd; ma=i; mb=j;
          }
        }
        if ((k==0)&&(xb>1.0e-20))
        { coef*=xb;
          for (j=k;j<eledim;++j)
            lumat[i*eledim+j]=origmat[i*eledim+j]/xb;
        }
        else if (k==0) coef=0.0f;
      }
      if (coef>1.0e-30)
      { if (mb!=k)
        { for (i=k;i<eledim;++i)
          { xa=lumat[i*eledim+k]; lumat[i*eledim+k]=lumat[i*eledim+mb];
            lumat[i*eledim+mb]=xa;
          }
          exchange=-exchange;
        }
        if (ma!=k)
        { for (j=k;j<eledim;++j)
          { xa=lumat[k*eledim+j]; lumat[k*eledim+j]=lumat[ma*eledim+j];
            lumat[ma*eledim+j]=xa;
          }
          exchange=-exchange;
        }
      }
      if (coef>1.0e-30)
      { xa=lumat[k*eledim+k];
        for (i=k+1;i<eledim;++i)
        { xb=lumat[i*eledim+k]; xc=-xb/xa;
          for (j=k;j<eledim;++j)
          { lumat[i*eledim+j]+=lumat[k*eledim+j]*xc;
          }
        }
      }
    }
    if ((error==0)&&(coef>1.0e-30))
    { sum=(double)exchange*coef;
      for (k=0;k<eledim;++k) sum*=lumat[k*eledim+k];
    }
    else sum=0.0f;
  }
  else if (error==0)
  { if ((exchange!=1)&&(exchange!=-1)) error=ludecomp();
    if (error==0)
    { sum=(double)exchange;
      for (i=0;i<eledim;++i) sum*=lumat[i*eledim+i];
    }
    else sum=0.0;
  }
  else sum=0.0;

  return(sum);
}// end of Matrix::determinant

//calculate inverse matrix
int Matrix::inverse(double *matout)
{ int i,j;
  double *vecin,*vecout;

  vecin=vecout=NULL;
  if (error==0)
  { vecin=new double [eledim]; vecout=new double [eledim];
    if ((!vecin)||(!vecout)) error=102;
  }

  if ((error==0)&&(exchange!=1)&&(exchange!=-1))
    error=ludecomp();
  for (j=0;(error==0)&&(j<eledim);++j)
  { for (i=0;(error==0)&&(i<eledim);++i)
    { if (i==j) vecin[i]=1.0;
      else vecout[i]=0.0;
    }
    error=lubacksub(vecin,vecout);
    for (i=0;i<eledim;++i) invmat[i*eledim+j]=vecout[i];
  }
  if ((error==0)&&(matout))
    for (i=0;i<elenum;++i) matout[i]=invmat[i];

  if (vecin) delete vecin; if (vecout) delete vecout;

  return(error);
} //end of Matrix::inverse

/*
----------------------------------------------------------------------
Solves the set of n=eledim linear equations AX=B. A is matrix,
B and X are vectors vecin and vecout here, respectively.
----------------------------------------------------------------------
*/
int Matrix::lubacksub(double *vecin,double *vecout)
{ int i,j,k,m;
  double xa;

  if ((error==0)&&(vecin)&&(vecout)&&(exchange!=1)&&(exchange!=-1))
    error=ludecomp();
  if ((error==0)&&(vecin)&&(vecout))
  { k=-1;
    for (i=0;i<eledim;++i) vecout[i]=vecin[i];
    for (i=0;i<eledim;++i)
    { m=pivot[i]; xa=vecout[m]; vecout[m]=vecout[i];
      if (k>=0) for (j=k;j<i;++j) xa-=lumat[i*eledim+j]*vecout[j];
      else if (xa!=0.0) k=i;
      vecout[i]=xa;
    }
    for (i=eledim-1;i>=0;--i)
    { xa=vecout[i];
      for (j=i+1;j<eledim;++j) xa-=lumat[i*eledim+j]*vecout[j];
      vecout[i]=xa/lumat[i*eledim+i];
    }
  }

  return(error);
} //end of Matrix::lubksb

/*
----------------------------------------------------------------------
LU decomposition of a rowwise permutation of lumat. pivot vector
is created to record the row permutation effected by the partial
pivoting. exchange equals +1 or -1 depending on whether the number
of row interchanges was even or odd, respectively.
----------------------------------------------------------------------
*/
int Matrix::ludecomp()
{ int i,j,k,m;
  double xa,xb,xc;
  double *vscale;

  vscale=NULL; m=0;

  if (error==0)
  { vscale=new double [eledim];
    if (!vscale) error=102;
  }

  if (error==0)
  { for (i=0;i<eledim;++i)
    { xa=0.0;
      for (j=0;j<eledim;++j)
      { xb=fabs(lumat[i*eledim+j]);
        if (xa<xb) xa=xb;
      }
      if (xa<1.0e-20) error=211;
      else vscale[i]=1.0f/xa;
    }
  }

  exchange=1;
  for (j=0;(error==0)&&(j<eledim);++j)
  { for (i=0;i<j;++i)
    { xa=lumat[i*eledim+j];
      for (k=0;k<i;++k) xa-=lumat[i*eledim+k]*lumat[k*eledim+j];
      lumat[i*eledim+j]=xa;
    }
    xb=0.0f;
    for (i=j;i<eledim;++i)
    { xa=lumat[i*eledim+j];
      for (k=0;k<j;++k) xa-=lumat[i*eledim+k]*lumat[k*eledim+j];
      lumat[i*eledim+j]=xa;
      xc=vscale[i]*fabs(xa);
      if (xb<xc)
      { xb=xc; m=i;
      }
    }
    if (j!=m)
    { for (k=0;k<eledim;++k)
      { xc=lumat[m*eledim+k];
        lumat[m*eledim+k]=lumat[j*eledim+k];
        lumat[j*eledim+k]=xc;
      }
      exchange=-exchange; vscale[m]=vscale[j];
    }
    pivot[j]=m; xc=lumat[j*eledim+j];
    if ((j!=eledim)&&(fabs(xc)>1.0e-20))
    { for (i=j+1;i<eledim;++i) lumat[i*eledim+j]/=xc;
    }
    else if (j!=eledim) error=211;
  }

  if (vscale) delete [] vscale;

  return(error);
}//end of Matrix::ludecomp

