/*
----------------------------------------------------------------------
  C++ classes designed for photoelectron diffraction software package
  Yufeng Chen, Michel A. Van Hove
  Lawrence Berkeley National Laboratory (LBNL), Berkeley, CA 94720
  Copyright (c) Van Hove Group 1997. All rights reserved.
----------------------------------------------------------------------

  Yufeng Chen, LBNL MS 2-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-4581 Fax (510) 486-5530 Email. ychen@LBL.gov

  M. A. Van Hove, LBNL MS 66-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-6160 Fax (510) 486-4995 Email. MAVanhove@LBL.gov
----------------------------------------------------------------------

List of classes

1. Matrix
Description: calculate error function
  erf(x)=(1/sqrt(pi))(integral)(exp(-t2)dt)(0-x)
Constructors and Destructor:
  Matrix() --- constructor
Member Functions:
  void init() --- initialization, must do before any other calls
  int getlength() --- return memory size needed by an object
  int export(char *dest,int length=0) --- export an object contents
    into memory buffer dest
  int import(char *source,int length=0) --- import an object contents
    from memory buffer source
  float getmemory() --- return memory used by an object
  int loadmatrix(int aeledim,double *tmat) --- load matrix of
    dimension aeledim, and elements in one-dimensional buffer
    tmat [0...aeledim*aeledim-1]
  double determinant(int lud=0) --- calculate determinant of the
    matrix loaded before. Choose lud=0 to do calculation by
    substitution method, and lud=1 to do calculation through
    LU decomposition
  int lubacksub(double *vecin,double *vecout) --- solves the set of
    n linear equations Ax=b, where A is the n x n matrix[0...n*n-1]
    loaded before, vecin[0...n-1] is input as the right-hand side
    vector b, vecout[0...n-1] is returned with the solution vector x.
  int inverse(double *matout=NULL) --- return inverse
    matrix to buffer matout[0...n*n-1], original matrix is loaded
    before.
Private Member Functions
  int allocation() --- prepare the memory space needed by an object
  int ludecomp() --- perform LU decomposition LU=A, where A is the
    original matrix loaded before, L is lower triangular (has elements
    only on the diagonal and below) and U is upper triangular
    (has elements only on the diagonal and above).
----------------------------------------------------------------------
*/

#ifndef __MATRIX_H
#define __MATRIX_H

class Matrix
{ private:
    int eledim,elenum,exchange,error;
    int *pivot;
    double *origmat,*lumat,*invmat;
  public:
    Matrix();
    ~Matrix();
    void init();
    int getlength();
    int export(char *dest,int length=0);
    int import(char *source,int length=0);
    float getmemory();
    int loadmatrix(int aeledim,double *tmat);
    double determinant(int lud=0);
    int lubacksub(double *vecin,double *vecout);
    int inverse(double *matout=NULL);
  private:
    int allocation();
    int ludecomp();
};

#endif //__MATRIX_H

