//Some special functions in quantum mechanics
#include <iostream.h>
#include <math.h>

#include "userutil.h"
#include "quantum.h"

Errorfunc::Errorfunc()
{ error=107; init();
}//end of Errorfunc::Errorfunc

Errorfunc::~Errorfunc()
{ if (acoef) delete [] acoef;
}//end of Errorfunc::~Errorfunc

void Errorfunc::init()
{ int k;
  if ((error==107)||(error==103))
  { acoef=NULL;
  }
  if (error==103)
  { ndata=100; mtaylor=7; eledim=64; estep=0.1f;
    if (error==0) k=sizeof(int); else k=sizeof(int);
    if (k<4) error=105;
    else
    { acoef=new float [ndata*eledim];
      if (!acoef) error=102;
    }
  }
  else if (error==106)
  { if (acoef)
    { acoef=new float [ndata*eledim];
      if (!acoef) error=102;
    }
  }
  if (error==106) error=0;
  else if (error==103) error=101;
  else if (error==107) error=103;

  return;
} //end of Errorfunc::init

int Errorfunc::getlength()
{ int ka;
  if (error==0)
  { ka=sizeof(Errorfunc)+sizeof(int);
    if (acoef) ka+=ndata*eledim*sizeof(float);
  }
  else ka=0;
  return(ka);
} //end of Errorfunc::getlength

int Errorfunc::export(char *dest,int length)
{ int ka,kb;
  if ((error==0)&&(dest))
  { ka=0; kb=getlength();
    if (length==0) length=kb;
    else if (length!=kb) ka=-1;
    kb=sizeof(int);
    if (ka>=0) ka=memorysend(dest,(char *)&length,ka,kb,length);
    kb=sizeof(Errorfunc);
    if (ka>=0) ka=memorysend(dest,(char *)this,ka,kb,length);
    kb=ndata*eledim*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)acoef,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  return(error);
} //end of Errorfunc::export

int Errorfunc::import(char *source,int length)
{ int ka,kb;

  if ((error==103)&&(source))
  { memorycopy((char *)&kb,source,sizeof(int));
    ka=sizeof(int);
    if ((length==0)&&(kb>0)) length=kb;
    else if ((length<0)||(kb<0)||(length!=kb)) ka=-1;
    kb=sizeof(Errorfunc);
    if (ka>=0) ka=memoryrec((char *)this,source,ka,kb,length);
    if (ka<0) error=901;
    else if (error==0)
    { error=106; init();
    }
    else error=754;
  }
  else ka=-1;
  if (error==0)
  { kb=ndata*eledim*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)acoef,source,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  return(error);
} //end of Errorfunc::import


/*
-----------------------------------------------------------------
coefficient index (j=k+1)
y=a0    +a1x+a2x2    +a3x3    +...+akxk      +...+a(n-1)x(n-1)+anxn

j  0     1   2   3   4   ...  n-2    n-1    n     n+1
x        0   1   2   3   ...  n-3    n-2    n-1   n
a  (n+2) a0  a1  a2  a3  ...  an-3   an-2   an-1  an
-----------------------------------------------------------------
*/
int Errorfunc::makecurve()
{ int i,j,k,m,n,t;
  float xa,xb,xc,sum;
  float *bcoef;
  const float pi=3.14159265f;

  bcoef=NULL;
  if (mtaylor>eledim-4) mtaylor=eledim-4;

  if ((error==101)||(error==0))
  { bcoef=new float [eledim*eledim];
    if (!bcoef) error=102;
    else error=0;
  }

  if (error==0)
  { for (j=0;j<eledim;++j)
    { if (j<2) for (k=0;k<eledim;++k) bcoef[j*eledim+k]=0.0f;
      else if (j==2)
      { n=j-1; bcoef[j*eledim]=(float)(n+2);
        for (k=1;k<eledim;++k)
        { t=k-1;
          if (t==0) xa=1.0f;
          else xa=0.0f;
          bcoef[j*eledim+k]=xa;
        }
      }
      else
      { n=j-1; bcoef[j*eledim]=(float)(n+2);
        for (k=1;k<eledim;++k)
        { t=k-1;
          if ((j>mtaylor+2)||(j>eledim-2)) xa=0.0f;
          else if (t>n) xa=0.0f;
          else if (n==0) xa=1.0f;
          else
          { xa=0.0f;
            if (t<n-1) xa+=(t+1)*bcoef[(j-1)*eledim+k+1];
            if (t>0) xa-=2*bcoef[(j-1)*eledim+k-1];
          }
          bcoef[j*eledim+k]=xa;
        }
      }
    }

    for (i=0;i<ndata;++i)
    { acoef[i*eledim]=(float)(mtaylor+2);
      xa=(float)(i*estep); xb=1.0;
      for (j=2;j<eledim;++j)
      { m=(int)(acoef[i*eledim]+0.1);
        xb*=(float)(j-1); xc=1.0;
        sum=0.0;
        if (j<m)
        { n=(int)(bcoef[j*eledim]+0.1);
          for (k=1;k<n;++k)
          { sum+=xc*bcoef[j*eledim+k];
            xc*=xa;
          }
          sum*=(float)(exp(-xa*xa)*2.0/sqrt(pi)/xb);
        }
        acoef[i*eledim+j]=sum;
      }
    }

    for (i=0;i<ndata;++i)
    { sum=0.0; xa=estep; xb=1.0;
      if (i>0)
      { m=(int)(acoef[(i-1)*eledim]+0.1);
        for (j=1;j<m;++j)
        { sum+=xb*acoef[(i-1)*eledim+j];
          xb*=xa;
        }
      }
      acoef[i*eledim+1]=sum;
    }
  }

  if (bcoef) delete [] bcoef;

  return(error);
}// end of Errorfunc::makecurve

float Errorfunc::getmemory()
{ float xa;
  if (error==0)
    xa=(float)getlength();
  else xa=-1.0f;
  return(xa);
} //end of Errorfunc::getmemory

float Errorfunc::ferf(float xa)
{ int i,j,m;
  float xb,xc,ya;

  if (error==0)
  { ya=0.0; xb=1.0;
    if (xa<0.0) xc=-xa; else xc=xa;
    if (estep<1.0e-5) i=0;
    else i=(int)(xc/estep+0.5);
    if (i<0) i=0; else if (i>ndata) i=ndata;
    xc-=i*estep;
    m=(int)(acoef[i*eledim]+0.1);
    for (j=1;j<m;++j)
    { ya+=xb*acoef[i*eledim+j];
      xb*=xc;
    }
  }

  return(ya);
}//end of Errorfunc::ferf

