/*
----------------------------------------------------------------------
  C++ classes designed for photoelectron diffraction software package
  Yufeng Chen, Michel A. Van Hove
  Lawrence Berkeley National Laboratory (LBNL), Berkeley, CA 94720
  Copyright (c) Van Hove Group 1997. All rights reserved.
----------------------------------------------------------------------

  Yufeng Chen, LBNL MS 2-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-4581 Fax (510) 486-5530 Email. ychen@LBL.gov

  M. A. Van Hove, LBNL MS 66-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-6160 Fax (510) 486-4995 Email. MAVanhove@LBL.gov
----------------------------------------------------------------------

List of classes

1. Errorfunc
Description: calculate error function
  erf(x)=(1/sqrt(pi))(integral)(exp(-t2)dt)(0-x)
Constructors and Destructor:
  Errorfunc() --- constructor
Member Functions:
  int init() --- initialization, return unzero if error occured
  float getmemory() --- get memory it took in unit of bytes
  float ferf(int xa) --- return erf(xa)
----------------------------------------------------------------------
*/

#ifndef __QUANTUM_H
#define __QUANTUM_H

class Errorfunc
{ private:
    int ndata,mtaylor,eledim,error;
    float estep;
    float *acoef;
  public:
    Errorfunc();
    ~Errorfunc();
    void init();
    int getlength();
    int export(char *dest,int length=0);
    int import(char *source,int length=0);
    int makecurve();
    float getmemory();
    float ferf(float xa);
};

#endif //__QUANTUM_H

