//program to calculate expectation energy
#include <iostream.h>
#include <strstream.h>

#include "userinfo.h"
#include "userutil.h"
#include "twoelec.h"

int parainput(int argc,char **argv,char *finaname,char *finbname,
  char *foutname,char *sysname)
{ int k,relax,error;

  error=0; relax=0;
  Textout conout;

  if (error==0)
  { conout.string("Enter name of input gaussian-output file or ",0,1);
    conout.string("      name of input expectation energy file",0,1);
    if (argc<=1) cin >> finaname;
    else
    { istrstream tempbuf(argv[1]);
      tempbuf >> finaname;
      conout.string(finaname,0,1);
    }
  }

  if (error==0)
  { ifstream fin(finaname,ios::in|ios::nocreate);
    if (!fin) error=201;
    else
    { getendline(fin,foutname,40);
      stringclear(foutname);
      k=stringcomp(foutname,"961",4);
      if (k==0) relax=1;
    }
  }

  k=2;
  if ((error==0)&&(relax==1))
  { conout.string(
      "Enter name of second input expectation energy file",0,1);
    if (argc<=k) cin >> finbname;
    else
    { istrstream tempbuf(argv[k]);
      tempbuf >> finbname;
      conout.string(finbname,0,1);
    }
    ++k;
  }
  else if (error==0) finbname[0]='\0';

  if (error==0)
  { if (relax==0) conout.string(
      "Enter name of output expectation energy file",0,1);
    else conout.string(
      "Enter name of output relaxation energy file",0,1);
    if (argc<=k) cin >> foutname;
    else
    { istrstream tempbuf(argv[k]);
      tempbuf >> foutname;
      conout.string(foutname,0,1);
    }
    ++k;
  }

  if (error==0)
  { conout.string("Enter name of the system (e.g. H2O): ");
    if (argc<=k) cin >> sysname;
    else
    { istrstream tempbuf(argv[k]);
      tempbuf >> sysname;
      conout.string(sysname,0,1);
    }
    ++k;
  }

  return(error);
} //end of parainput

int main(int argc,char **argv)
{ int help,nbasis,ndata,relax,error;
  float xa;
  char finaname[80],finbname[80],foutname[80],sysname[80];

  error=0; finaname[0]=finbname[0]=foutname[0]=sysname[0]='\0';
  Userinfo userinfo; userinfo.init();
  userinfo.loadparameter(1.12f,"reps",
    "Y Chen, G Zhuang, PN Ross, MA Van Hove and CS Fadley",
    "Calculation of relaxation energy of photoemission spectra");
  Textout conout;
  conout.string(userinfo.usermsg,0,1);
  if (userinfo.version==0.0) error=102;

  if ((error==0)&&(argc>1))
  { help=stringcomp(argv[1],"help",5);
    if (help!=0) help=stringcomp(argv[1],"?",2);
    if (help!=0) help=stringcomp(argv[1],"\\?",3);
    if (help!=0) help=stringcomp(argv[1],"/?",3);
  }
  else help=1;

  if ((error==0)&&(help!=0))
    error=parainput(argc,argv,finaname,finbname,foutname,sysname);

  if ((error==0)&&(help!=0))
  { Twoelec twoelec; twoelec.init();
    error=twoelec.loadcurve(finaname,finbname,sysname);
    if (error==0) relax=twoelec.getrelax();
    else relax=0;
    if ((error==0)&&(relax==0))
    { xa=twoelec.getmemory();
      xa=xa*1.0e-6f+0.1f;
      conout.string("This job took ");
      conout.floating(xa,7.1f);
      conout.string(" megabytes memory",0,2);

      nbasis=twoelec.getnumbasis();
      ndata=twoelec.getnumpoint();
    }
    if (error==0) error=twoelec.maskinnershell();
    if ((error==0)&&(relax==0)) error=twoelec.sumintegral();
    if (error==0) error=twoelec.savecurve(foutname,userinfo.usermsg);
  }
  else if (error==0)
  { conout.string("Usage: reps input_gaussian output_expect ",0,16);
    conout.string("system_name",0,1);
    conout.string("   or: reps first_expect second_expect ");
    conout.string("output_relax system_name",0,2);
    conout.string("       input_gaussian: name of ");
    conout.string("gaussian-output file as reps input",0,1);
    conout.string("       output_expect:  name of output ");
    conout.string("expectation value file",0,1);
    conout.string("       system_name:    name of the system",0,1);
    conout.string("       first_expect:   name of first input ");
    conout.string("expectation value file",0,1);
    conout.string("       second_expect:  name of second input ");
    conout.string("expectation value file",0,1);
    conout.string("       output_relax:   name of output ");
    conout.string("relaxation energy file",0,1);
    conout.string("Example: reps goh2o.txt evh2o.txt H2O",0,16+1);
    conout.string("     or: reps evo.txt evfion.txt reo.txt O",0,1);
  }

  Errorinfo errorinfo(error);
  conout.string(errorinfo.errmsg,0,16+1);

  return(error);
} //end of main


/*
----------------------------------------------------------------------
REPS version history


REPS expectation value program was started on July 7, 1997, using
ANSI C++ object oriented programming (OOP) technique.

Version 1.00, July 12, 1997
  First version, working on PC, Sun workstation, Cray J90

Version 1.01, July 14, 1997
  Improve the user interface with a help message if user ask help by
  entering     reps help   or reps ?

Version 1.02, July 15, 1997
  Removed one bug in reading gaussian input file with more than 5
  alpha or beta electrons

Version 1.03, July 15, 1997
  Add a time stamp at bottom of the output file, recording the
  job processor time and execution time

Version 1.04, July 20, 1997
  Add charge and gaussian basis information into the output file

Version 1.10, July 25, 1997
  Add one more column in reps output file, which is the expectation
  value excluding the inner shell (1s shell in this version)
  contribution

Version 1.11, July 30, 1997
  Add one feature for myself (not for user), calculating relaxation
  energy by inputing two expectation files
  Undocumented Feature: calculate relaxation energy
    Usage: reps first_expect second_expect output_relax system_name
    Example: reps evo.txt evfion.txt reo.txt O_atom
  where, first_expect is the first input expectation file for atom Z,
  and second_expect is the second input expectation file for its
  equivalent core ion Z+1

Version 1.12, August 5, 1997
  Add process report for large job. In reading or summing, there are
  now process indication letters '-' displaying on console

----------------------------------------------------------------------
*/

