//Some special functions in quantum mechanics
#include <iostream.h>
#include <math.h>
#include <iomanip.h>

#include "userutil.h"
#include "matrix.h"
#include "jobtime.h"
#include "ryspoly.h"

Ryspoly::Ryspoly()
{ error=107; init();
}//end of Ryspoly::Ryspoly

Ryspoly::~Ryspoly()
{ if (ryselem) delete [] ryselem; if (ryscoef) delete [] ryscoef;
  if (talpha) delete [] talpha; if (walpha) delete [] walpha;
  if (ualpha) delete [] ualpha;
  if (casym) delete [] casym; if (jobtime) delete jobtime;
}//end of Ryspoly::~Ryspoly

void Ryspoly::init()
{ 
  if ((error==107)||(error==103))
  { datatype=964; ntaylor=nrys=ndata=none=0;
    ryselem=ryscoef=talpha=walpha=ualpha=casym=NULL;
  }
  if (error==103)
  { datatype=964; ntaylor=nrys=ndata=none=0;
    ryselem=ryscoef=talpha=walpha=ualpha=casym=NULL;
  }
  else if (error==106)
  { if (ryselem)
    { ryselem=new double [nelem*2*nrys*ntaylor];
      if (!ryselem) error=102;
    }
    if (ryscoef)
    { ryscoef=new double [4*nrys*nrys];
      if (!ryscoef) error=102;
    }
    if (talpha)
    { talpha=new double [ndata*nrys*nrys];
      if (!talpha) error=102;
    }
    if (walpha)
    { walpha=new double [ndata*nrys*nrys];
      if (!walpha) error=102;
    }
    if (ualpha)
    { ualpha=new double [ndata*nrys*nrys];
      if (!ualpha) error=102;
    }
    if (casym)
    { casym=new double [8*nrys*nrys];
      if (!casym) error=102;
    }
  }
  else if (error==107)
  { jobtime=new Jobtime;
    if (!jobtime) error=102;
    else jobtime->reset();
  }
  if (error==106) error=0;
  else if (error==103) error=101;
  else if (error==107) error=103;

  return;
} //end of Ryspoly::init

int Ryspoly::getlength()
{ int ka;
  if (error==0)
  { ka=sizeof(Ryspoly)+sizeof(int);
    if (ryselem)
      ka+=nelem*2*nrys*ntaylor*sizeof(double);
    if (ryscoef) ka+=4*nrys*nrys*sizeof(double);
    if (talpha) ka+=ndata*nrys*nrys*sizeof(double);
    if (walpha) ka+=ndata*nrys*nrys*sizeof(double);
    if (ualpha) ka+=ndata*nrys*nrys*sizeof(double);
    if (casym) ka+=8*nrys*nrys*sizeof(double);
  }
  else ka=0;
  return(ka);
} //end of Ryspoly::getlength

int Ryspoly::export(char *dest,int length)
{ int ka,kb;
  if ((error==0)&&(dest))
  { ka=0; kb=getlength();
    if (length==0) length=kb;
    else if (length!=kb) ka=-1;
    kb=sizeof(int);
    if (ka>=0) ka=memorysend(dest,(char *)&length,ka,kb,length);
    kb=sizeof(Ryspoly);
    if (ka>=0) ka=memorysend(dest,(char *)this,ka,kb,length);
    kb=nelem*2*nrys*ntaylor*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)ryselem,ka,kb,length);
    kb=4*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)ryscoef,ka,kb,length);
    kb=ndata*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)talpha,ka,kb,length);
    kb=ndata*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)walpha,ka,kb,length);
    kb=ndata*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)ualpha,ka,kb,length);
    kb=8*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memorysend(dest,(char *)casym,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  return(error);
} //end of Ryspoly::export

int Ryspoly::import(char *source,int length)
{ int ka,kb;
  void *temp;

  temp=(void *)jobtime;

  if ((error==103)&&(source))
  { memorycopy((char *)&kb,source,sizeof(int));
    ka=sizeof(int);
    if ((length==0)&&(kb>0)) length=kb;
    else if ((length<0)||(kb<0)||(length!=kb)) ka=-1;
    kb=sizeof(Ryspoly);
    if (ka>=0) ka=memoryrec((char *)this,source,ka,kb,length);
    if (ka<0) error=901;
    else if (error==0)
    { error=106; init();
    }
    else error=754;
  }
  else ka=-1;
  if (error==0)
  { kb=nelem*2*nrys*ntaylor*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)ryselem,source,ka,kb,length);
    kb=4*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)ryscoef,source,ka,kb,length);
    kb=ndata*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)talpha,source,ka,kb,length);
    kb=ndata*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)walpha,source,ka,kb,length);
    kb=ndata*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)ualpha,source,ka,kb,length);
    kb=8*nrys*nrys*sizeof(double);
    if (ka>=0) ka=memoryrec((char *)casym,source,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  jobtime=(Jobtime *)temp;

  return(error);
} //end of Ryspoly::import

int Ryspoly::makelement()
{ int i,j,k;
  double xa,xb,xc,xd,xe;
  double *bcoef;

  bcoef=NULL;

  if (error==0)
  { bcoef=new double [nrys+nrys+ntaylor*nelem];
    if (!bcoef) error=102;
    else error=0;
  }

  if (error==0)
  { for (j=0;j<nrys+nrys+ntaylor*nelem;++j)
      bcoef[j]=double(1.0/(j+j+1.0));

    for (i=0;i<nelem-2;++i)
    { xa=(double)(i*estep);
      for (j=0;j<nrys+nrys+(nelem-2-i)*ntaylor;++j)
      { xb=1.0; xc=1.0; xe=0.0;
        for (k=0;k<ntaylor;++k)
        { if (k>0)
          { xb*=(double)(-k); xc*=estep;
          }
          xd=bcoef[j+k]/xb;
          xe+=xc*xd;
          if (j<nrys+nrys) ryselem[i*2*nrys*ntaylor+j*ntaylor+k]=xd;
        }
        bcoef[j]=xe;
      }
    }
  }

  if (error==0)
  { xe=1.0;
    for (j=0;j<nrys+nrys;++j)
    { ryselem[(nelem-1)*2*nrys*ntaylor+j*ntaylor]=xe;
      xe*=(j+j+1.0)/2.0;
    }
  }

  if (bcoef) delete [] bcoef;

  return(error);
} //end of Ryspoly::makelement

int Ryspoly::makesubelement(double xa)
{ int i,j,k;
  double xb,xc,xd,xe;

  if ((error==0)&&(ryselem)&&(xa>=0.0)&&(xa<25.0+estep)&&
    (fabs(xa-prxa)>1.0e-3))
  { prxa=xa;
    i=(int)(xa/estep+0.5);
    if (i<0) i=0; else if (i>nelem-3) i=nelem-3;
    xb=xa-i*estep;
    for (j=0;j<nrys+nrys;++j)
    { xe=0.0; xc=1.0;
      for (k=0;k<ntaylor;++k)
      { xd=ryselem[i*2*nrys*ntaylor+j*ntaylor+k];
        xe+=xc*xd; xc*=xb;
      }
      ryselem[(nelem-2)*2*nrys*ntaylor+j*ntaylor]=xe;
    }
  }

  return(error);
} //end of Ryspoly::makesubelement

double Ryspoly::getelement(int bnrys,double xa)
{ double ya;

  if ((error==0)&&(ryselem)&&(bnrys>=0)&&(bnrys<nrys+nrys)&&
    (xa>=0.0)&&(xa<25.0))
  { if (fabs(xa-prxa)>1.0e-3) error=makesubelement(xa);
    if (error==0)
      ya=ryselem[(nelem-2)*2*nrys*ntaylor+bnrys*ntaylor];
    else ya=0.0;
  }
  else ya=0.0;

  return(ya);
}//end of Ryspoly::getelement

int Ryspoly::makeroots(int anrys)
{ int i,j,k,ma,mb,mc,md,rasym;
  double xa,xb,xc,xd,xbeg,ya,yb,yc,rstep;
  double *tmat;
  const double pi=3.141592653590;

  tmat=NULL;

  if ((error==101)||(error==0))
  { ndata=253; nelem=253; ntaylor=10; prxa=-1.0;
    xstep=(double)(25.0/(ndata-3.0));
    estep=(double)(25.0/(nelem-3.0));
    if (anrys==0) nrys=8;
    else if (anrys<2) nrys=2;
    else if (anrys>10) nrys=10;
    else nrys=anrys;
    none=anrys;
    if (none<0) none=0;
    else if (none>nrys-1) none=nrys-1;

    if (ryselem) delete [] ryselem; if (ryscoef) delete [] ryscoef;
    if (talpha) delete [] talpha; if (walpha) delete [] walpha;
    if (ualpha) delete [] ualpha;
    ryselem=new double [nelem*2*nrys*ntaylor];
    ryscoef=new double [4*nrys*nrys];
    talpha=new double [ndata*nrys*nrys];
    walpha=new double [ndata*nrys*nrys];
    ualpha=new double [ndata*nrys*nrys];
    casym=new double [8*nrys*nrys];
    k=nrys*nrys;
    if (k<ndata*3) k=ndata*3;
    if (k<100) k=100;
    tmat=new double [k];
    if ((!talpha)||(!walpha)||(!ualpha)||(!ryselem)||(!ryscoef)||
      (!tmat)||(!casym)) error=102;
    else error=0;
  }

  if (error==0) error=makelement();

  Matrix matrix;
  if (error==0) matrix.init();
  md=10; rasym=0;
  for (i=-1;(error==0)&&(i<ndata-2);++i)
  { if (i<0) mc=(nelem-1)*2*nrys*ntaylor;
    else
    { xa=(double)(i*xstep);
      error=makesubelement(xa);
      mc=(nelem-2)*2*nrys*ntaylor;
    }

    for (j=0;(error==0)&&(j<nrys);++j)
    { if (j==0)
      { rasym=0;
        xa=ryselem[mc+j*ntaylor];
        ryscoef[j*nrys]=1.0/sqrt(xa);
      }
      else if (rasym==0)
      { for (ma=0;ma<j;++ma)
        { for (mb=0;mb<j;++mb)
            tmat[ma*j+mb]=ryselem[mc+(ma+mb)*ntaylor];
        }
        matrix.loadmatrix(j,tmat);
        xa=matrix.determinant();
        for (ma=0;ma<j+1;++ma)
        { for (mb=0;mb<j+1;++mb)
            tmat[ma*(j+1)+mb]=ryselem[mc+(ma+mb)*ntaylor];
        }
        matrix.loadmatrix(j+1,tmat);
        xb=matrix.determinant();
        ryscoef[(j+2*nrys)*nrys]=xa;
        ryscoef[(j+2*nrys)*nrys+1]=xb;
        xa*=xb;
        if (xa<0.0) xa=-xa;
        xa=sqrt(xa);
        if (xa<1.0e-100) xa=0.0;
        else xa=1.0/xa;

        for (k=0;k<j+1;++k)
        { for (ma=0;ma<j;++ma)
          { for (mb=0;mb<j;++mb)
            { if (ma<k)
                tmat[ma*j+mb]=ryselem[mc+(ma+mb)*ntaylor];
              else
                tmat[ma*j+mb]=ryselem[mc+(ma+mb+1)*ntaylor];
            }
          }
          matrix.loadmatrix(j,tmat);
          xb=matrix.determinant();
          if (((k+j)&1)!=0) xb=-xb;
          ryscoef[j*nrys+k]=xa*xb;
        }
        xa=ryscoef[j*nrys];
        if (i>md) xb=ryscoef[(j+3*nrys)*nrys];
        else xb=xa;
        if (fabs(xa+xb)>1.0e-20) xc=fabs((xa-xb)/(xa+xb));
        else xc=1.0;
        if (xc>0.01) rasym=1; else rasym=0;
      }
      for (k=0;k<nrys;++k)
      { if ((k>j)||(rasym!=0)) ryscoef[j*nrys+k]=0.0;
        ryscoef[(j+3*nrys)*nrys+k]=ryscoef[j*nrys+k];
      }
    }

    xbeg=0.0;
    for (j=0;(error==0)&&(j<nrys);++j)
    { for (k=0;(error==0)&&(k<nrys);++k)
      { if (i>=0) ma=i*nrys*nrys+j*nrys+k;
        else ma=j*nrys+k;
        xa=fabs(ryscoef[j*nrys]);
        if (((xa<1.0e-20)&&(i>md)&&(i>0))||(i==ndata-3)) rasym=1;
        else rasym=0;
        if (k>=j) talpha[ma]=0.0;
        else if (rasym!=0)
        { xb=casym[(j+3*nrys)*nrys+k];
          if (xb<0.5)
          { xb=(i-2.0)*xstep; xc=(i-1.0)*xstep;
            yb=talpha[ma-2*nrys*nrys]; yc=talpha[ma-nrys*nrys];
            casym[(j+3*nrys)*nrys+k]=1.0;
            casym[(j+nrys)*nrys+k]=(xc*xc*yc-xb*xb*yb+
              (xb*sqrt(xb)-xc*sqrt(xc))*casym[j*nrys+k])/(xc-xb);
            casym[(j+2*nrys)*nrys+k]=xc*xc*yc-
              xc*casym[(j+nrys)*nrys+k]-xc*sqrt(xc)*casym[j*nrys+k];
          }
          talpha[ma]=casym[j*nrys+k]/sqrt(i*xstep)+
            casym[(j+nrys)*nrys+k]/(i*xstep)+
            casym[(j+2*nrys)*nrys+k]/(i*i*xstep*xstep);
        }
        else if ((j==1)&&(k<j))
        { xa=ryscoef[j*nrys+1]; xb=ryscoef[j*nrys];
          if (fabs(xa)<1.0e-20) xc=0.0;
          else xc=-xb/xa;
          if (xc<0.0) error=901;
          else xc=sqrt(xc);
          talpha[ma]=xc;
        }
        else if ((j==2)&&(k<j))
        { xa=ryscoef[j*nrys+2]; xb=ryscoef[j*nrys+1];
          xc=ryscoef[j*nrys];
          xd=xb*xb-4.0f*xa*xc;
          if (xd<0.0) error=901;
          else xd=sqrt(xd);
          if (k==0) xd=-xb-xd;
          else xd=-xb+xd;
          if (fabs(xa)<1.0e-20) xd=0.0;
          else xd=0.5f*xd/xa;
          if (xd<0.0) error=901;
          else xd=sqrt(xd);
          talpha[ma]=xd;
        }
        else if (k<j)
        { rstep=0.25f/(double)(j+1);
          if (k==0) xbeg=xa=0.0;
          else xa=xbeg;
          xb=xa;
          ya=yb=fryspoly(j,xa);
          while ((error==0)&&(xb<10.0)&&
            (((ya>0.0)&&(yb>0.0))||((ya<0.0)&&(yb<0.0))))
          { xa=xb; ya=yb; xb+=rstep; if (xb>10.0) xb=10.0;
            yb=fryspoly(j,xb);
          }
          xbeg=xb+rstep;
          while ((error==0)&&(fabs(xa-xb)>0.01))
          { xc=(xa+xb)*0.5; yc=fryspoly(j,xc);
            if (((ya>0.0)&&(yc>0.0))||((ya<0.0)&&(yc<0.0)))
            { xa=xc; ya=yc;
            }
            else
            { xb=xc; yb=yc;
            }
          }
          xc=xd=(xa+xb)*0.5; yc=fryspoly(j,xc); rstep=xc-xa;
          xa=fabs(xa)+fabs(xb)+fabs(xc);
          if ((xa>1.0e-20)&&(rstep>1.0e-5))
          { xa=rstep/xa; ya*=xa; yb*=xa; yc*=xa;
            xc=yc; xb=0.5f*(yb-ya)/rstep;
            xa=0.5f*(yb+ya-yc-yc)/rstep/rstep;
            if (fabs(xa)>1.0e-5)
            { xc=xb*xb-4.0f*xa*xc;
              if (xc<0.0) error=901;
              else xc=sqrt(xc);
              ya=0.5f*(-xb-xc)/xa; yb=0.5f*(-xb+xc)/xa;
            }
            else if (fabs(xb)>1.0e-5) ya=yb=-xc/xb;
            else if (fabs(xc)>1.0e-5) error=901;
            else ya=yb=0.0;
            if ((ya>-rstep-1.0e-5)&&(ya<rstep+1.0e-5)) xa=xd+ya;
            else if ((yb>-rstep-1.0e-5)&&(yb<rstep+1.0e-5)) xa=xd+yb;
            else error=901;
          }
          else xa=xd;
          talpha[ma]=xa;
        }
        else talpha[ma]=0.0;

        xa=talpha[ma]; xb=0.0;
        if ((error==0)&&((xa<0.0)||(xa>10.0))) error=901;
        else if ((error==0)&&(xa<1.0e-5)) xb=0.0;
        else if ((error==0)&&(k<j)&&(rasym!=0))
        { xb=casym[(j+7*nrys)*nrys+k];
          if (xb<0.5)
          { xb=(i-2.0)*xstep; xc=(i-1.0)*xstep;
            yb=walpha[ma-2*nrys*nrys]; yc=walpha[ma-nrys*nrys];
            casym[(j+7*nrys)*nrys+k]=1.0;
            casym[(j+5*nrys)*nrys+k]=(xc*xc*yc-xb*xb*yb+
              (xb*sqrt(xb)-xc*sqrt(xc))*casym[(j+4*nrys)*nrys+k]);
            casym[(j+6*nrys)*nrys+k]=xc*xc*yc-
              xc*casym[(j+5*nrys)*nrys+k]-xc*sqrt(xc)*
              casym[(j+4*nrys)*nrys+k];
          }
          xb=casym[(j+4*nrys)*nrys+k]/sqrt(i*xstep)+
            casym[(j+5*nrys)*nrys+k]/(i*xstep)+
            casym[(j+6*nrys)*nrys+k]/(i*i*xstep*xstep);
        }
        else if ((error==0)&&(k<j))
        { for (mb=0;mb<j;++mb)
          { xc=fryspoly(mb,xa); xb+=xc*xc;
          }
          if (xb<1.0e-5) error=901;
          else xb=1.0f/xb;
          error=0;
        }
        if (error==0) walpha[ma]=xb;
      }
    }

    if ((error==0)&&(i<0))
    { for (k=0;k<8*nrys*nrys;++k) casym[k]=0.0;
      for (j=0;(error==0)&&(i<0)&&(j<nrys);++j)
      { for (k=0;(error==0)&&(k<nrys);++k)
        { ma=j*nrys+k;
          casym[ma]=talpha[ma]; casym[4*nrys*nrys+ma]=walpha[ma];
        }
      }
    }
  }

  for (i=0;i<ndata-2;++i)
  { for (j=0;j<nrys;++j)
    { for (k=0;k<nrys;++k)
      { ma=i*nrys*nrys+j*nrys+k;
        xa=talpha[ma];
        if (xa<1.0-1.0e-5) xb=xa*xa/(1.0-xa*xa);
        else xb=0.0;
        ualpha[ma]=xb;
      }
    }
  }

  if (tmat) delete [] tmat;

  return(error);
}// end of Ryspoly::makeroots

double Ryspoly::asymptotic(int ka,int ma,double *xalpha)
{ int i,j,eledim;
  double xa,xb,xc;
  double *tmat;

  tmat=NULL; eledim=4;

  if (error==0)
  { tmat=new double [2*eledim*eledim];
    if (!tmat) error=102;
  }

  Matrix matrix;
  if ((error==0)&&(ka>eledim))
  { matrix.init();
    for (i=0;i<eledim;++i)
    { xa=(ka-i-1.0)*xstep; xb=1.0/sqrt(xa); xc=xb;
      for (j=0;j<eledim;++j)
      { tmat[i*eledim+j]=xc; xc*=xb;
      }
    }
    error=matrix.loadmatrix(eledim,tmat);
    xa=ka*xstep; xb=1.0/sqrt(xa); xc=xb;
    for (i=0;(error==0)&&(i<eledim);++i)
    { tmat[eledim+eledim+i]=xalpha[ma-(i+1)*nrys*nrys];
      tmat[eledim+i]=xc; xc*=xb;
    }
    matrix.lubacksub(tmat+eledim+eledim,tmat);
    xa=0.0;
    for (i=0;i<eledim;++i)
    { xa+=tmat[i]*tmat[eledim+i];
    }
  }
  else xa=0.0;

  return(xa);
} //end of Ryspoly::asymptotic

double Ryspoly::fryspoly(int mrys,double xt)
{ int i;
  double xa,xb,ya;

  if ((error==0)&&(mrys<nrys)&&(xt>-1.0e-5)&&(xt<10.0))
  { xa=1.0; xb=xt*xt; ya=0.0;
    for (i=0;i<mrys+1;++i)
    { ya+=ryscoef[mrys*nrys+i]*xa;
      xa*=xb;
    }
  }
  else ya=0.0;

  return(ya);
}// end of Ryspoly::fryspoly

float Ryspoly::getmemory()
{ float xa;
  if (error==0) xa=(float)getlength();
  else xa=-1.0;
  return(xa);
} //end of Ryspoly::getmemory

double Ryspoly::gettalpha(int mrys,int mroot,double xa)
{ int i,j,k;
  double xb,ya;

  if ((error==0)&&(mrys<nrys)&&(mroot<mrys)&&(xa>=0.0)&&(xa<25.0))
  { i=(int)(xa/xstep);
    if (i<0) i=0;
    else if (i>ndata-4) i=ndata-4;
    xb=xa/xstep-(double)i;
    j=i*nrys*nrys+mrys*nrys+mroot;
    k=j+nrys*nrys;
    ya=talpha[j]+xb*(talpha[k]-talpha[j]);
  }
  else if ((error==0)&&(mrys<nrys)&&(mroot<mrys)&&(xa>0.0)&&(xa<100.0))
  { ya=casym[mrys*nrys+mroot]/sqrt(xa)+
      casym[(mrys+nrys)*nrys+mroot]/xa+
      casym[(mrys+2*nrys)*nrys+mroot]/xa/xa;
  }
  else ya=0.0;

  return(ya);
} //end of Ryspoly::gettalpha

double Ryspoly::getwalpha(int mrys,int mroot,double xa)
{ int i,j,k;
  double xb,ya;

  if ((error==0)&&(mrys<nrys)&&(mroot<mrys)&&(xa>=0.0)&&(xa<25.0))
  { i=(int)(xa/xstep);
    if (i<0) i=0;
    else if (i>ndata-4) i=ndata-4;
    xb=xa/xstep-(double)i;
    j=i*nrys*nrys+mrys*nrys+mroot;
    k=j+nrys*nrys;
    ya=walpha[j]+xb*(walpha[k]-walpha[j]);
  }
  else if ((error==0)&&(mrys<nrys)&&(mroot<mrys)&&(xa>0.0)&&(xa<100.0))
  { ya=casym[(mrys+4*nrys)*nrys+mroot]/sqrt(xa)+
      casym[(mrys+5*nrys)*nrys+mroot]/xa+
      casym[(mrys+6*nrys)*nrys+mroot]/xa/xa;
  }
  else ya=0.0;

  return(ya);
} //end of Ryspoly::getwalpha

double Ryspoly::getualpha(int mrys,int mroot,double xa)
{ int i,j,k;
  double xb,ya;

  if ((error==0)&&(mrys<nrys)&&(mroot<mrys)&&(xa>=0.0)&&(xa<25.0))
  { i=(int)(xa/xstep);
    if (i<0) i=0;
    else if (i>ndata-4) i=ndata-4;
    xb=xa/xstep-(double)i;
    j=i*nrys*nrys+mrys*nrys+mroot;
    k=j+nrys*nrys;
    ya=ualpha[j]+xb*(ualpha[k]-ualpha[j]);
  }
  else if ((error==0)&&(mrys<nrys)&&(mroot<mrys)&&(xa>0.0)&&(xa<100.0))
  { ya=casym[mrys*nrys+mroot]/sqrt(xa)+
      casym[(mrys+nrys)*nrys+mroot]/xa+
      casym[(mrys+2*nrys)*nrys+mroot]/xa/xa;
    if (ya<1.0-1.0e-5) ya=ya*ya/(1.0-ya*ya);
    else ya=0.0;
  }
  else ya=0.0;

  return(ya);
} //end of Ryspoly::getualpha

int Ryspoly::getalphas(int mrys,double xa,double *xtalpha,
  double *xwalpha,double *xualpha)
{ int i,j,k,ma,mb,mc;
  double xb;

  i=(int)(xa/xstep);
  if (i<0) i=0;
  else if (i>ndata-4) i=ndata-4;
  xb=xa/xstep-(double)i;
  mb=i*nrys*nrys+mrys*nrys; mc=mb+nrys*nrys;
  for (ma=0;ma<mrys;++ma)
  { j=mb+ma; k=mc+ma;
    *xtalpha=talpha[j]+xb*(talpha[k]-talpha[j]);
    *xwalpha=walpha[j]+xb*(walpha[k]-walpha[j]);
    *xualpha=ualpha[j]+xb*(ualpha[k]-ualpha[j]);
  }

  return(error);
} //end of Ryspoly::getalphas

int Ryspoly::getalphas(int mrys,float xa,float *xtalpha,
  float *xwalpha,float *xualpha)
{ int i,j,k,ma,mb,mc;
  float xb;

  i=(int)(xa/xstep);
  if (i<0) i=0;
  else if (i>ndata-4) i=ndata-4;
  xb=(float)(xa/xstep-(double)i);
  mb=i*nrys*nrys+mrys*nrys; mc=mb+nrys*nrys;
  for (ma=0;ma<mrys;++ma)
  { j=mb+ma; k=mc+ma;
    *xtalpha=(float)(talpha[j]+xb*(talpha[k]-talpha[j]));
    *xwalpha=(float)(walpha[j]+xb*(walpha[k]-walpha[j]));
    *xualpha=(float)(ualpha[j]+xb*(ualpha[k]-ualpha[j]));
  }

  return(error);
} //end of Ryspoly::getalphas

int Ryspoly::saveroots(char *filename,char *usermessage)
{ int i,j,k,ma,mb,np,begrow;
  float xa,xb,xc;

  jobtime->stop();

  if (error==0)
  { datatype=964;
    Textout fileout(filename);
    error=fileout.geterror();
    if (error==0)
    { i=0; np=101;
      if (none==1) begrow=9;
      else if (none==2) begrow=10;
      else begrow=11;
      while (usermessage[i]!='\0')
      { if ((usermessage[i]=='\n')||(usermessage[i]=='\r')) ++begrow;
        ++i;
      }
      fileout.integer(datatype,6); fileout.integer(begrow,6);
      fileout.integer(0,6); fileout.space(5);
      fileout.string("datakind beginning-row linenumbers",0,1);
      fileout.string(usermessage,0,1);
      fileout.string(" roots and their quadrature weight factors ");
      fileout.string(" of Rys polynomial",0,2);

      fileout.string(" nrys="); fileout.integer(nrys-1,2,1);
      fileout.string(" column format:",0,1);
      fileout.string("   ndata nrys-min nrys-max nroots//",0,1);
      fileout.string("    x   nrys   talpha-1   walpha-1",0,1);
      if ((none==0)||(none>1))
        fileout.string("               talpha-2   walpha-2",0,1);
      if (none==3)
        fileout.string("               talpha-3   walpha-3",0,1);
      else if ((none==0)||(none>3))
      { fileout.string("        ....   talpha-");
        fileout.integer(nrys-1);
        fileout.string("   walpha-"); fileout.integer(nrys-1,0,1);
      }
      if (none==0)
      { ma=1; mb=nrys;
      }
      else
      { ma=none; mb=ma+1;
      }
      if (mb>nrys) mb=nrys;
      fileout.integer(np,8); fileout.integer(ma,8);
      fileout.integer(mb-1,8);
      fileout.integer((mb-ma)*(ma+mb-1),8,1);
      for (i=0;i<np;++i)
      { xa=(float)(i*25.0f/(np-1.0));
        fileout.floating(xa,7.2f,256);
        for (j=ma;j<mb;++j)
        { if (j>ma) fileout.space(7);
          fileout.integer(j,3);
          for (k=0;k<j;++k)
          { xb=(float)gettalpha(j,k,(double)xa);
            xc=(float)getwalpha(j,k,(double)xa);
            if (k>0) fileout.space(10);
            fileout.floating(xb,12.7f,256);
            fileout.floating(xc,16.6f,512+1);
          }
        }
      }
      if (error==0) error=fileout.geterror();
      if (error==0)
      { error=jobtime->timestamp(fileout);
        if (error!=0) error+=(datatype<<16);
      }
    }
    else error+=(datatype<<16);
  }

  return(error);
} //end of Ryspoly::saveroots

