#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

#include "userutil.h"
#include "jobtime.h"
#include "twoelec.h"

Twoelec::Twoelec()
{ error=107; init();
} //end of Twoelec::Twoelec

Twoelec::~Twoelec()
{ if (sysname) delete [] sysname; if (basisname) delete [] basisname;
  if (orbital) delete [] orbital; if (twoeadd) delete [] twoeadd;
  if (twoeint) delete [] twoeint; if (expecval) delete [] expecval;
  if (eigenval) delete [] eigenval; if (mocoef) delete [] mocoef;
  if (innermask) delete [] innermask; if (jobtime) delete jobtime;
} //end of Twoelec::~Twoelec

void Twoelec::init()
{ if ((error==107)||(error==103))
  { datatype=nbasis=nalpha=nbeta=ngauss=ndata=charge=multiplicity=0;
    sysname=basisname=orbital=NULL; twoeadd=NULL;
    twoeint=expecval=eigenval=mocoef=innermask=NULL;
  }
  if (error==103)
  { sysname=new char [80];
    if (!sysname) error=102;
    basisname=new char [80];
    if (!basisname) error=102;
  }
  else if (error==106)
  { if (sysname)
    { sysname=new char [80];
      if (!sysname) error=102;
    }
    if (basisname)
    { basisname=new char [80];
      if (!basisname) error=102;
    }
    if (orbital)
    { orbital=new char [nbasis*10];
      if (!orbital) error=102;
    }
    if (twoeadd)
    { twoeadd=new int [ndata];
      if (!twoeadd) error=102;
    }
    if (twoeint)
    { twoeint=new float [ndata];
      if (!twoeint) error=102;
    }
    if (expecval)
    { expecval=new float [nbasis*16];
      if (!expecval) error=102;
    }
    if (eigenval)
    { eigenval=new float [nbasis*2];
      if (!eigenval) error=102;
    }
    if (mocoef)
    { mocoef=new float [nbasis*nbasis*2];
      if (!mocoef) error=102;
    }
    if (innermask)
    { innermask=new float [nbasis*nbasis*4];
      if (!innermask) error=102;
    }
  }
  else if (error==107)
  { jobtime=new Jobtime;
    if (!jobtime) error=102;
    else jobtime->reset();
  }
  if (error==106) error=0;
  else if (error==103) error=101;
  else if (error==107) error=103;

  return;
} //end of Twoelec::init

int Twoelec::getlength()
{ int ka;
  if (error==0)
  { ka=sizeof(Twoelec)+80*2*sizeof(char);
    if (orbital) ka+=nbasis*10*sizeof(char);
    if (twoeadd) ka+=ndata*sizeof(int);
    if (twoeint) ka+=ndata*sizeof(float);
    if (expecval) ka+=nbasis*16*sizeof(float);
    if (eigenval) ka+=nbasis*2*sizeof(float);
    if (mocoef) ka+=nbasis*nbasis*2*sizeof(float);
    if (innermask) ka+=nbasis*nbasis*4*sizeof(float);
  }
  else ka=0;
  return(ka);
} //end of Twoelec::getlength

int Twoelec::export(char *dest,int length)
{ int ka,kb;
  if ((error==0)&&(dest))
  { ka=0; kb=getlength();
    if (length==0) length=kb;
    else if (length!=kb) ka=-1;
    kb=sizeof(int);
    if (ka>=0) ka=memorysend(dest,(char *)&length,ka,kb,length);
    kb=sizeof(Twoelec);
    if (ka>=0) ka=memorysend(dest,(char *)this,ka,kb,length);
    kb=80;
    if (ka>=0) ka=memorysend(dest,sysname,ka,kb,length);
    kb=80;
    if (ka>=0) ka=memorysend(dest,basisname,ka,kb,length);
    kb=nbasis*10;
    if (ka>=0) ka=memorysend(dest,orbital,ka,kb,length);
    kb=ndata*sizeof(int);
    if (ka>=0) ka=memorysend(dest,(char *)twoeadd,ka,kb,length);
    kb=ndata*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)twoeint,ka,kb,length);
    kb=nbasis*16*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)expecval,ka,kb,length);
    kb=nbasis*2*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)eigenval,ka,kb,length);
    kb=nbasis*nbasis*2*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)mocoef,ka,kb,length);
    kb=nbasis*nbasis*4*sizeof(float);
    if (ka>=0) ka=memorysend(dest,(char *)innermask,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  return(error);
} //end of Twoelec::export

int Twoelec::import(char *source,int length)
{ int ka,kb;
  void *temp;
  
  temp=(void *)jobtime;
  if ((error==103)&&(source))
  { memorycopy((char *)&kb,source,sizeof(int));
    ka=sizeof(int);
    if ((length==0)&&(kb>0)) length=kb;
    else if ((length<0)||(kb<0)||(length!=kb)) ka=-1;
    kb=sizeof(Twoelec);
    if (ka>=0) ka=memoryrec((char *)this,source,ka,kb,length);
    if (ka<0) error=901;
    else if (error==0)
    { error=106; init();
    }
    else error=754;
  }
  else ka=-1;
  if (error==0)
  { kb=80;
    if (ka>=0) ka=memoryrec(sysname,source,ka,kb,length);
    kb=80;
    if (ka>=0) ka=memoryrec(basisname,source,ka,kb,length);
    kb=nbasis*10;
    if (ka>=0) ka=memoryrec(orbital,source,ka,kb,length);
    kb=ndata*sizeof(int);
    if (ka>=0) ka=memoryrec((char *)twoeadd,source,ka,kb,length);
    kb=ndata*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)twoeint,source,ka,kb,length);
    kb=nbasis*16*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)expecval,source,ka,kb,length);
    kb=nbasis*2*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)eigenval,source,ka,kb,length);
    kb=nbasis*nbasis*2*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)mocoef,source,ka,kb,length);
    kb=nbasis*nbasis*4*sizeof(float);
    if (ka>=0) ka=memoryrec((char *)innermask,source,ka,kb,length);
    if ((ka<0)||(ka!=length)) error=901;
  }
  jobtime=(Jobtime *)temp;

  return(error);
} //end of Twoelec::import

int Twoelec::getnumpoint()
{ int k;
  if (error==0) k=ndata;
  else k=0;
  return(k);
} //end of Twoelec::getnumpoint

int Twoelec::getnumbasis()
{ int k;
  if (error==0) k=nbasis;
  else k=0;
  return(k);
} //end of Twoelec::getnumbasis

int Twoelec::getrelax()
{ int k;
  if (error==0) k=relax;
  else k=0;
  return(k);
} //end of Twoelec::getrelax

float Twoelec::getmemory()
{ float xa;
  if (error==0) xa=(float)getlength();
  else xa=-1.0f;
  return(xa);
} //end of Twoelec::getmemory

int Twoelec::loadcurve(char *finaname,char *finbname,char *isysname)
{ int i,j,k,begrow,linenum,memsize,jobcount;
  float xa,xb,xc,xd,xe;
  char *membuf;
  const float hartree=27.2116f;

  membuf=NULL; memsize=256; datatype=962;
  relax=ndata=linenum=jobcount=0;
  if ((error==101)||(error==0))
  { membuf=new char [memsize];
    if (!membuf) error=102;
    else error=0;
  }

  Textout conout;
  if (error==0) stringcopy(sysname,isysname,40);

  if (error==0)
  { ifstream fin(finaname,ios::in|ios::nocreate);
    if (!fin) error=201+(datatype<<16);
    else
    { fin >> membuf;
      k=stringcomp(membuf,"961",4);
      if (k==0)
      { relax=1; datatype=961;
      }
      else relax=0;
      if (!fin) error=201+(datatype<<16);
    }


    if ((error==0)&&(relax==0))
    { xa=xb=xc=0.0;
      
      k=1;
      while ((fin)&&(k!=0))
      { getendline(fin,membuf,memsize);
        stringclear(membuf);
        k=stringcomp(membuf,"symbolic z matrix",16);
      }
      if (fin)
      { fin >> membuf >> membuf >> xa >> membuf >> membuf >> xb;
        if (!fin) error=204+(datatype<<16);
        else if ((xa>=-10)&&(xa<=10)&&(xb>=1)&&(xb<=10))
        { charge=(int)xa; multiplicity=(int)xb;
        }
        else error=207+(datatype<<16);
      }

      k=1;
      while ((fin)&&(k!=0))
      { getendline(fin,membuf,memsize);
        stringcopy(membuf+50,membuf,50);
        stringclear(membuf);
        k=stringcomp(membuf,"standard basis",15);
      }
      if (fin)
      { for (i=50;i<100;++i) if (membuf[i]==':') break;
        stringcopy(basisname,membuf+i+2,24);
        for (i=0;i<24;++i) if ((basisname[i]=='\0')||
          (basisname[i]=='\n')||(basisname[i]=='\r')) break;
        basisname[i]='\0';
      }
      else error=207+(datatype<<16);

      k=1;
      while ((fin)&&(k!=0))
      { getendline(fin,membuf,memsize);
        stringclear(membuf);
        k=stringcomp(membuf,"regular integral format",23);
      }
      if (fin)
      { skipendline(fin);
        fin >> xa >> membuf >> membuf >> xb >> membuf >> membuf;
        nbasis=(int)xa; ngauss=(int)xb;
        fin >> xa >> membuf >> membuf >> xb >> membuf >> membuf;
        nalpha=(int)xa; nbeta=(int)xb;
      }

      k=1;
      while ((fin)&&(k!=0))
      { getendline(fin,membuf,memsize);
        stringclear(membuf);
        k=stringcomp(membuf,"dumping two electron integrals",30);
      }
      if (fin)
      { fin >> membuf; skipendline(fin); skipendline(fin);
        fin >> membuf >> xa >> membuf >> xb; skipendline(fin);
        ndata=(int)xb;
      }
      if (!fin) error=207+(datatype<<16);
    }

    if ((error==0)&&(relax==0))
    { if (error==0) k=sizeof(int); else k=sizeof(int);
      if ((error==0)&&(k<4)&&((nbasis>12)||(ndata>15000))) error=702;
      else if ((error==0)&&((nbasis>128)||(ndata>5.0e6))) error=702;
      else
      { orbital=new char [nbasis*10]; twoeadd=new int [ndata];
        twoeint=new float [ndata]; expecval=new float [nbasis*16];
        eigenval=new float [nbasis*2];
        mocoef=new float [nbasis*nbasis*2];
        innermask=new float [nbasis*nbasis*4];
        if ((!orbital)||(!twoeadd)||(!twoeint)||(!expecval)||
          (!eigenval)||(!mocoef)||(!innermask)) error=102;
      }
    }
    else ndata=0;

    if ((error==0)&&(relax==0))
    { conout.string("Reading ",0,16); conout.integer(ndata,9);
      conout.string(" guassian two-electron integrals ... ",0,1);
    }

    jobcount=0;
    for (i=0;(error==0)&&(i<ndata);++i)
    { fin >> membuf >> xa >> membuf >> xb >> membuf >> xc
        >> membuf >> xd >> membuf >> xe;
      if ((xa>=1)&&(xa<=nbasis)&&(xb>=1)&&(xb<=nbasis)&&
        (xc>=1)&&(xc<=nbasis)&&(xd>=1)&&(xd<=nbasis))
      { twoeadd[i]=(int)((xa-1)+(xb-1)*nbasis+(xc-1)*nbasis*nbasis+
         (xd-1)*nbasis*nbasis*nbasis);
        twoeint[i]=xe*hartree;
      }
      else error=207+(datatype<<16);
      k=(i+1)*53/ndata-jobcount;
      if ((error==0)&&(k>0))
      { conout.charfill('-',k); conout.flush(); jobcount+=k;
      }
    }
    conout.newline();
    if ((error==0)&&(!fin)) error=204+(datatype<<16);

    if ((error==0)&&(relax==0))
    { k=1;
      while ((fin)&&(k!=0))
      { getendline(fin,membuf,memsize);
        stringclear(membuf);
        if (multiplicity<=1)
          k=stringcomp(membuf,"molecular orbital coefficients",29);
        else
          k=stringcomp(membuf,"alpha molecular orbital coeffi",29);
      }
      if (fin)
      { for (i=0;(error==0)&&(i<nbasis);i+=5)
        { skipendline(fin); skipendline(fin);
          skipbytes(fin,21);
          for (k=i;(k<i+5)&&(k<nbasis);++k)
          { fin >> xa; eigenval[k]=xa*hartree;
          }
          skipendline(fin);
          for (j=0;j<nbasis;++j)
          { skipbytes(fin,9); fin >> (orbital+j*10+3);
            if ((orbital[j*10+3]<'0')||(orbital[j*10+3]>'9'))
            { for (k=0;k<3;++k)
              { if (((orbital[j*10+k+3]>='A')&&
                  (orbital[j*10+k+3]<='Z'))||((orbital[j*10+k+3]>='a')
                  &&(orbital[j*10+k+3]<='z')))
                  orbital[j*10+k]=orbital[j*10+k+3];
                else orbital[j*10+k]=' ';
              }
              fin >> (orbital+j*10+3);
            }
            else
            { for (k=0;k<3;++k)
              { if (j>0) orbital[j*10+k]=orbital[(j-1)*10+k];
                else orbital[j*10+k]=' ';
              }
            }
            for (k=i;(k<i+5)&&(k<nbasis);++k)
            { fin >> mocoef[k*nbasis+j];
            }
            skipendline(fin);
          }
        }
      }
      if (!fin) error=207+(datatype<<16);
    }

    if ((error==0)&&(relax==0)&&(multiplicity>1))
    { k=1;
      while ((fin)&&(k!=0))
      { getendline(fin,membuf,memsize);
        stringclear(membuf);
        k=stringcomp(membuf,"beta molecular orbital coeffic",29);
      }
      if (fin)
      { for (i=nbasis;(error==0)&&(i<nbasis*2);i+=5)
        { skipendline(fin); skipendline(fin);
          skipbytes(fin,21);
          for (k=i;(k<i+5)&&(k<nbasis*2);++k)
          { fin >> xa; eigenval[k]=xa*hartree;
          }
          skipendline(fin);
          for (j=0;j<nbasis;++j)
          { skipbytes(fin,21);
            for (k=i;(k<i+5)&&(k<nbasis*2);++k)
            { fin >> mocoef[k*nbasis+j];
            }
            skipendline(fin);
          }
        }
      }
      if (!fin) error=204+(datatype<<16);
    }
    else if ((error==0)&&(relax==0))
    { for (k=nbasis;k<nbasis*2;++k)
      { eigenval[k]=eigenval[k-nbasis];
        for (j=0;j<nbasis;++j)
          mocoef[k*nbasis+j]=mocoef[(k-nbasis)*nbasis+j];
      }
    }

    if ((error==0)&&(relax==1))
    { fin >> xa >> xb; begrow=(int)xa; linenum=(int)xb;
      for (i=0;i<begrow-1;++i) getendline(fin,membuf,memsize);
      if (linenum==0)
      { fin >> xa >> xb; getendline(fin,membuf,memsize);
        k=(int)xa; nbasis=(int)(xb*2.0+10);
        if (!fin) error=201+(datatype<<16);
        else if (k!=2) error=207+(datatype<<16);
      }
      else if (linenum>0) nbasis=linenum*4+10;
      else error=207+(datatype<<16);
      if (error==0)
      { expecval=new float [nbasis*16];
        eigenval=new float [nbasis*2];
        if ((!expecval)||(!eigenval)) error=102;
      }
    }

    if ((error==0)&&(relax==1))
    { if (linenum==0)
      { fin >> xa >> xb; getendline(fin,membuf,memsize);
        nalpha=(int)xb;
      }
      else nalpha=linenum;
      if (!fin) error=207+(datatype<<16);
      else if (nalpha>nbasis/2) error=207+(datatype<<16);
      else
      { for (i=0;i<nalpha;++i)
        { fin >> xa >> xb >> xc >> xd; getendline(fin,membuf,memsize);
          eigenval[i]=xb; expecval[i]=xc;
          expecval[nbasis*2+i]=xd;
        }
      }
      if ((error==0)&&(linenum==0))
      { fin >> xa >> xb; getendline(fin,membuf,memsize);
        nbeta=(int)xb;
        if (nbeta>nbasis/2) error=207+(datatype<<16);
        else
        { for (i=0;i<nbeta;++i)
          { fin >> xa >> xb >> xc >> xd; getendline(fin,membuf,memsize);
            eigenval[nbasis+i]=xb; expecval[nbasis+i]=xc;
            expecval[nbasis*3+i]=xd;
          }
        }
      }
      else if (error==0)
      { nbeta=nalpha;
        for (i=0;i<nbeta;++i)
        { eigenval[nbasis+i]=eigenval[i];
          expecval[nbasis+i]=expecval[i];
          expecval[nbasis*3+i]=expecval[nbasis*2+i];
        }
      }
    }
    if ((error==0)&&(!fin)) error=207+(datatype<<16);
  }

  if ((error==0)&&(relax==1))
  { ifstream fin(finbname,ios::in|ios::nocreate);
    if (!fin) error=201+(datatype<<16);
    else
    { fin >> xa; k=(int)xa;
      if (!fin) error=201+(datatype<<16);
      else if (k!=961) error=201+(datatype<<16);
    }
    if (error==0)
    { fin >> xa >> xb; begrow=(int)xa; linenum=(int)xb;
      for (i=0;i<begrow-1;++i) getendline(fin,membuf,memsize);
      if (linenum==0)
      { fin >> xa >> xb; getendline(fin,membuf,memsize);
        k=(int)xa;
        if (k!=2) error=207+(datatype<<16);
        else
        { fin >> xa >> xb; getendline(fin,membuf,memsize);
          k=(int) xb;
        }
      }
      else k=linenum;
      if (nalpha>k) nalpha=k;
      for (i=0;i<k;++i)
      { fin >> xa >> xb >> xc >> xd; getendline(fin,membuf,memsize);
        if (i<nalpha)
        { expecval[nbasis*4+i]=xc;
          expecval[nbasis*6+i]=xd;
        }
      }
    }
    if ((error==0)&&(linenum==0))
    { fin >> xa >> xb; getendline(fin,membuf,memsize);
      k=(int)xb; if (k<nbeta) nbeta=k;
      for (i=0;i<k;++i)
      { fin >> xa >> xb >> xc >> xd; getendline(fin,membuf,memsize);
        if (i<nbeta)
        { expecval[nbasis*5+i]=xc;
          expecval[nbasis*7+i]=xd;
        }
      }
    }
    else if (error==0)
    { nbeta=nalpha;
      for (i=0;i<nbeta;++i)
      { expecval[nbasis*5+i]=expecval[nbasis*4+i];
        expecval[nbasis*7+i]=expecval[nbasis*6+i];
      }
    }
  }

  if (membuf) delete [] membuf;

  return(error);
} //end of Twoelec::loadcurve

