#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

#include "userutil.h"
#include "jobtime.h"
#include "twoelec.h"

float Twoelec::getmocoef(int elevel,int basis)
{ float xa;

  if ((error==0)&&(basis>=0)&&(basis<nbasis)&&(elevel>=0)&&
    (elevel<nbasis*2))
  { xa=mocoef[elevel*nbasis+basis];
  }
  else xa=0.0f;

  return(xa);
} //end of Twoelec::getmocoef

float Twoelec::getintegral(int ka)
{ float xa;

  if ((error==0)&&(ka>=0)&&(ka<ndata))
    xa=twoeint[ka];
  else xa=0.0f;

  return(xa);
} //end of Twoelec::getintegral

int Twoelec::getintadd(int ka)
{ int kb;

  if ((error==0)&&(ka>=0)&&(ka<ndata))
    kb=twoeadd[ka];
  else kb=0;

  return(kb);
} //end of Twoelec::getintadd

int Twoelec::maskinnershell()
{ int i,j,ka,kb,m;
  float xa,xb,xc;

  Textout conout;
  if ((error==0)&&(relax==0))
  { for (i=0;(error==0)&&(i<nbasis*2);++i)
    { for (j=0;j<nbasis*2;++j)
      { if (((i<nalpha)||((i>=nbasis)&&(i<nbasis+nbeta)))&&
          ((j<nalpha)||((j>=nbasis)&&(j<nbasis+nbeta))))
        { xc=1.0f;
          for (kb=0;kb<nbasis;++kb)
          { xb=getmocoef(j,kb);
            if (((xb>0.5)||(xb<-0.5))&&(orbital[kb*10+3]=='1'))
            { for (ka=0;ka<nbasis;++ka)
              { xa=getmocoef(i,ka);
                if ((xa>0.5)||(xa<-0.5))
                { m=stringcomp((orbital+ka*10),(orbital+kb*10),3);
                  if (m==0)
                  { xc=0.0f; ka=kb=nbasis;
                  }
                }
              }
            }
          }
        }
        else xc=0.0f;
        innermask[i*nbasis*2+j]=xc;
      }
    }
  }
  else if (error==0)
  { for (i=0;(i<nalpha)||(i<nbeta);++i)
    { expecval[nbasis*8+i]=(expecval[nbasis*4+i]-expecval[i])*0.5f;
      expecval[nbasis*9+i]=(expecval[nbasis*5+i]-expecval[nbasis+i])*
        0.5f;
      expecval[nbasis*10+i]=(expecval[nbasis*6+i]-
        expecval[nbasis*2+i])*0.5f;
      expecval[nbasis*11+i]=(expecval[nbasis*7+i]-
        expecval[nbasis*3+i])*0.5f;
      xa=(-eigenval[i]-expecval[nbasis*10+i]);
      xb=(-eigenval[nbasis+i]-expecval[nbasis*11+i]);
      if (xa<0.0) xa=0.0f;
      if (xb<0.0) xb=0.0f;
      expecval[nbasis*12+i]=xa; expecval[nbasis*13+i]=xb;
    }
    conout.string("calculated "); conout.integer(nalpha+nbeta,3);
    conout.string(" points",0,1);
  }

  return(error);
} //end of Twoelec::maskinnershell

int Twoelec::sumintegral()
{ int ia,ib,ic,id,ja,jb,k,ma,mb,ngamma,jobcount;
  float xa,xb,xc;
  float *stat,*suma,*sumb;

  stat=suma=sumb=NULL; jobcount=0;
  if ((error==0)&&(relax==0))
  { stat=new float [50];
    suma=new float [nbasis*2]; sumb=new float [nbasis*2];
    if ((!stat)||(!suma)||(!sumb)) error=102;
  }

  Textout conout;
  if ((error==0)&&(relax==0))
  { conout.string("Summing ");
    conout.integer(ndata,8);
    conout.string(" two-electron integrals for ");
    conout.integer(nbasis,4);
    conout.string(" basis functions ...",0,1);

    for (k=0;k<50;++k) stat[k]=0.0f;
    stat[41]=1.0f; stat[42]=2.0f; stat[43]=4.0f;
    stat[44]=4.0f; stat[45]=8.0f;
    stat[21]=(float)nbasis;
    stat[22]=(float)(nbasis*(nbasis-1)/2);
    stat[23]=(float)(nbasis*(nbasis-1)/2);
    stat[24]=(float)(nbasis*nbasis*(nbasis-1)/2);
    stat[25]=(float)(nbasis*(nbasis-1)*(nbasis*nbasis-nbasis-2)/8);
    for (k=1;k<=5;++k)
    { stat[30+k]=stat[20+k]*stat[40+k];
      stat[20]+=stat[20+k]; stat[30]+=stat[30+k];
    }
    stat[26]=(float)(nbasis*(nbasis+1)*(nbasis*nbasis+nbasis+2)/8);
    stat[36]=(float)(nbasis*nbasis*nbasis*nbasis);
    if ((nbasis<40)&&((stat[20]!=stat[26])||(stat[30]!=stat[36])))
      error=901;

  }

  if (nalpha>nbeta) ngamma=nalpha; else ngamma=nbeta;

  for (ma=0;(error==0)&&(relax==0)&&(ma<nbasis*2);++ma)
  { for (mb=0;mb<nbasis*2;++mb) suma[mb]=sumb[mb]=0.0f;
    if ((ma<ngamma)||((multiplicity>1)&&(ma>=nbasis)&&
      (ma<nbasis+nbeta)))
    { for (mb=0;mb<nbasis*2;++mb)
      { if ((mb<ngamma)||((multiplicity>1)&&(mb>=nbasis)&&
          (mb<nbasis+nbeta)))
        { for (ja=0;(error==0)&&(ja<ndata);++ja)
          { jb=twoeadd[ja]; xa=twoeint[ja];
            ia=jb%nbasis; ib=(jb/nbasis)%nbasis;
            ic=(jb/nbasis/nbasis)%nbasis;
            id=(jb/nbasis/nbasis/nbasis);
            if ((ia==ib)&&(ic==id)&&(ib==ic))
            { suma[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              if ((ma==0)&&(mb==0))
              { stat[1]+=1.0; stat[11]+=1.0;
              }
            }
            else if ((ia==ib)&&(ic==id)&&(ia!=ic))
            { suma[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              suma[mb]+=oneintegral(ma,mb,ic,id,ia,ib,xa);
              if ((ma==0)&&(mb==0))
              { stat[2]+=1.0; stat[12]+=2.0;
              }
            }
            else if ((ia!=ib)&&(ic!=id)&&(((ia==ic)&&(ib==id))||
              ((ia==id)&&(ib==ic))))
            { suma[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              suma[mb]+=oneintegral(ma,mb,ia,ib,id,ic,xa);
              suma[mb]+=oneintegral(ma,mb,ib,ia,ic,id,xa);
              suma[mb]+=oneintegral(ma,mb,ib,ia,id,ic,xa);
              if ((ma==0)&&(mb==0))
              { stat[3]+=1.0; stat[13]+=4.0;
              }
            }
            else if (((ia==ib)&&(ic!=id))||((ia!=ib)&&(ic==id)))
            { suma[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              suma[mb]+=oneintegral(ma,mb,ib,ia,id,ic,xa);
              suma[mb]+=oneintegral(ma,mb,ic,id,ia,ib,xa);
              suma[mb]+=oneintegral(ma,mb,id,ic,ib,ia,xa);
              if ((ma==0)&&(mb==0))
              { stat[4]+=1.0; stat[14]+=4.0;
              }
            }
            else if ((ia!=ib)&&(ic!=id)&&(((ia!=ic)&&(ia!=id))||
              ((ib!=ic)&&(ib!=id))))
            { suma[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              suma[mb]+=oneintegral(ma,mb,ia,ib,id,ic,xa);
              suma[mb]+=oneintegral(ma,mb,ib,ia,ic,id,xa);
              suma[mb]+=oneintegral(ma,mb,ib,ia,id,ic,xa);
              suma[mb]+=oneintegral(ma,mb,ic,id,ia,ib,xa);
              suma[mb]+=oneintegral(ma,mb,ic,id,ib,ia,xa);
              suma[mb]+=oneintegral(ma,mb,id,ic,ia,ib,xa);
              suma[mb]+=oneintegral(ma,mb,id,ic,ib,ia,xa);
              if ((ma==0)&&(mb==0))
              { stat[5]+=1.0; stat[15]+=8.0;
              }
            }
            else error=901;

            if ((error!=0)||((ma<nbasis)&&(mb>=nbasis))||
              ((ma>=nbasis)&&(mb<nbasis))) continue;

            k=ib; ib=id; id=k;
            if ((ia==ib)&&(ic==id)&&(ib==ic))
            { sumb[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
            }
            else if ((ia==ib)&&(ic==id)&&(ia!=ic))
            { sumb[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              sumb[mb]+=oneintegral(ma,mb,ic,id,ia,ib,xa);
            }
            else if ((ia!=ib)&&(ic!=id)&&(((ia==ic)&&(ib==id))||
              ((ia==id)&&(ib==ic))))
            { sumb[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              sumb[mb]+=oneintegral(ma,mb,ia,ib,id,ic,xa);
              sumb[mb]+=oneintegral(ma,mb,ib,ia,ic,id,xa);
              sumb[mb]+=oneintegral(ma,mb,ib,ia,id,ic,xa);
            }
            else if (((ia==ib)&&(ic!=id))||((ia!=ib)&&(ic==id)))
            { sumb[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              sumb[mb]+=oneintegral(ma,mb,ib,ia,id,ic,xa);
              sumb[mb]+=oneintegral(ma,mb,ic,id,ia,ib,xa);
              sumb[mb]+=oneintegral(ma,mb,id,ic,ib,ia,xa);
            }
            else if ((ia!=ib)&&(ic!=id)&&(((ia!=ic)&&(ia!=id))||
              ((ib!=ic)&&(ib!=id))))
            { sumb[mb]+=oneintegral(ma,mb,ia,ib,ic,id,xa);
              sumb[mb]+=oneintegral(ma,mb,ia,ib,id,ic,xa);
              sumb[mb]+=oneintegral(ma,mb,ib,ia,ic,id,xa);
              sumb[mb]+=oneintegral(ma,mb,ib,ia,id,ic,xa);
              sumb[mb]+=oneintegral(ma,mb,ic,id,ia,ib,xa);
              sumb[mb]+=oneintegral(ma,mb,ic,id,ib,ia,xa);
              sumb[mb]+=oneintegral(ma,mb,id,ic,ia,ib,xa);
              sumb[mb]+=oneintegral(ma,mb,id,ic,ib,ia,xa);
            }
            else error=901;
          }
        }
      }
    }

    if (error==0)
    { xa=xb=0.0f;
      for (mb=0;mb<nbasis*2;++mb)
      { if ((mb==ma)||((ma>=nalpha)&&(ma<nbasis))||(ma>=nbasis+nbeta))
          continue;
        else if (multiplicity>1) xc=suma[mb]-sumb[mb];
        else if ((ma<nbasis)&&(mb<nbasis)) xc=suma[mb]-sumb[mb];
        else if (ma<nbasis) xc=suma[mb-nbasis];
        else xc=0.0f;
        xa+=xc;
        if (innermask[ma*nbasis*2+mb]>0.0) xb+=xc;
      }
      if ((ma<nalpha)||(multiplicity>1))
      { expecval[ma]=xa; expecval[ma+nbasis*2]=xb;
      }
      else if (ma>=nbasis)
      { expecval[ma]=expecval[ma-nbasis];
        expecval[ma+nbasis*2]=expecval[ma+nbasis];
      }
    }
    k=(ma+1)*68/nbasis/2-jobcount;
    if ((error==0)&&(k>0))
    { conout.charfill('-',k); conout.flush(); jobcount+=k;
    }
  }
  conout.newline();

  if ((error==0)&&(relax==0))
  { for (k=1;k<=5;++k)
    { stat[0]+=stat[k]; stat[10]+=stat[10+k];
    }
    for (k=0;k<=5;++k)
    { stat[k]=stat[k]/stat[20+k]*100.0f;
      stat[10+k]=stat[10+k]/stat[30+k]*100.0f;
    }

    conout.string("statistics (test only, not for user) ",0,1);
    for (k=1;k<=5;++k) conout.floating(stat[k],10.2f,256);
    conout.newline();
    for (k=1;k<=5;++k) conout.floating(stat[10+k],10.2f,256);
    conout.newline();
  }

  if (stat) delete [] stat;
  if (suma) delete [] suma; if (sumb) delete [] sumb;

  return(error);
} //end of Twoelec::sumintegral

float Twoelec::oneintegral(int ma,int mb,int ia,int ib,int ic,int id,
  float integral)
{ float xa,xb,xc,xd,xf;

  if (error==0)
  { xa=getmocoef(ma,ia); xb=getmocoef(ma,ib);
    xc=getmocoef(mb,ic); xd=getmocoef(mb,id);
    xf=xa*xb*xc*xd*integral;
  }
  else xf=0.0f;

  return(xf);
} //end of Twoelec::oneintegral

int Twoelec::savecurve(char *filename,char *usermessage)
{ int i,begrow;

  jobtime->stop();

  if ((error==0)&&(relax==0))
  { datatype=961;
    Textout fileout(filename);
    error=fileout.geterror();
    if (error==0)
    { i=0; begrow=13;
      while (usermessage[i]!='\0')
      { if ((usermessage[i]=='\n')||(usermessage[i]=='\r')) ++begrow;
        ++i;
      }
      fileout.integer(datatype,6); fileout.integer(begrow,6);
      fileout.integer(0,6); fileout.space(5);
      fileout.string("datakind beginning-row linenumbers",0,1);
      fileout.string(usermessage,0,1);
      fileout.string("   "); fileout.string(sysname);
      fileout.string(" expectation values",0,2);

      fileout.string("   charge = "); fileout.integer(charge,2);
      fileout.string("   multiplicity = ");
      fileout.integer(multiplicity,2);
      fileout.string(" basis = "); fileout.string(basisname,0,1);
      fileout.string("   number of basis functions = ");
      fileout.integer(nbasis,3);
      fileout.string(" number of occupied orbitals = ");
      fileout.integer(nalpha+nbeta,3,1);
      fileout.string("   number of alpha electrons = ");
      fileout.integer(nalpha,3);
      fileout.string(" number of beta  electrons   = ");
      fileout.integer(nbeta,3,1);
      fileout.string("   number of non-zero integrals = ");
      fileout.integer(ndata,9,2);
      fileout.string(
        "   expectation values of alpha and beta electrons",0,1);
      fileout.string(
        "     parameters: electron_kind number_of_electrons",0,1);
      fileout.string(
        "     columns: number eigenvalue expectationa ");
      fileout.string("expectationb (unit: eV)",0,1);
      fileout.integer(2,6); fileout.integer(nalpha+nbeta,6);
      fileout.string("        nelectron npoint",0,1);
      fileout.integer(1,4); fileout.integer(nalpha,6);
      fileout.string(
        "          alpha electron -------------------",0,1);
      for (i=0;i<nalpha;++i)
      { fileout.integer(i+1,4);
        fileout.floating(eigenval[i],14.4f,512);
        fileout.floating(expecval[i],14.4f,512);
        fileout.floating(expecval[i+nbasis*2],14.4f,512);
        fileout.newline();
      }
      fileout.integer(2,4); fileout.integer(nbeta,6);
      fileout.string(
        "          beta  electron -------------------",0,1);
      for (i=0;i<nbeta;++i)
      { fileout.integer(i+1,4);
        fileout.floating(eigenval[nbasis+i],14.4f,512);
        fileout.floating(expecval[nbasis+i],14.4f,512);
        fileout.floating(expecval[nbasis*3+i],14.4f,512);
        fileout.newline();
      }
      if (error==0) error=fileout.geterror();
      if (error==0)
      { error=jobtime->timestamp(fileout);
        if (error!=0) error+=(datatype<<16);
      }
    }
    else error+=(datatype<<16);
  }
  else if (error==0)
  { datatype=963;
    Textout fileout(filename);
    error=fileout.geterror();
    if (error==0)
    { i=0; begrow=8;
      while (usermessage[i]!='\0')
      { if ((usermessage[i]=='\n')||(usermessage[i]=='\r')) ++begrow;
        ++i;
      }
      fileout.integer(datatype,6); fileout.integer(begrow,6);
      fileout.integer(0,6); fileout.space(5);
      fileout.string("datakind beginning-row linenumbers",0,1);
      fileout.string(usermessage,0,1);
      fileout.string("   "); fileout.string(sysname);
      fileout.string(" relaxation energies",0,2);

      fileout.string(
        "   relaxation energies of alpha and beta electrons",0,1);
      fileout.string(
        "     parameters: electron_kind number_of_electrons",0,1);
      fileout.string(
        "     columns: number eigen relaxa ");
      fileout.string("relaxb binding (unit: eV)",0,1);
      fileout.integer(2,6); fileout.integer(nalpha+nbeta,6);
      fileout.string("        nelectron npoint",0,1);
      fileout.integer(1,4); fileout.integer(nalpha,6);
      fileout.string(
        "          alpha electron -------------------",0,1);
      for (i=0;i<nalpha;++i)
      { fileout.integer(i+1,4);
        fileout.floating(eigenval[i],14.4f,512);
        fileout.floating(expecval[nbasis*8+i],8.2f,256);
        fileout.floating(expecval[nbasis*10+i],8.2f,256);
        fileout.floating(expecval[nbasis*12+i],14.4f,512);
        fileout.newline();
      }
      fileout.integer(2,4); fileout.integer(nbeta,6);
      fileout.string(
        "          beta  electron -------------------",0,1);
      for (i=0;i<nbeta;++i)
      { fileout.integer(i+1,4);
        fileout.floating(eigenval[nbasis+i],14.4f,512);
        fileout.floating(expecval[nbasis*9+i],8.2f,256);
        fileout.floating(expecval[nbasis*11+i],8.2f,256);
        fileout.floating(expecval[nbasis*13+i],14.4f,512);
        fileout.newline();
      }
      if (error==0) error=fileout.geterror();
    }
    else error+=(datatype<<16);
  }

  return(error);
} //end of Twoelec::savecurve


